@@ -53,8 +53,8 @@ from x2go.backends.info import X2goServerSessionInfo as _X2goServerSessionInfo
 from x2go.backends.info import X2goServerSessionList as _X2goServerSessionList
 from x2go.backends.proxy import X2goProxy as _X2goProxy
 
-from x2go.monkey_patch_paramiko import monkey_patch_paramiko
-monkey_patch_paramiko()
+import x2go._paramiko
+x2go._paramiko.monkey_patch_paramiko()
 
 def _rerewrite_blanks(cmd):
     """\
@@ -117,6 +117,7 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
                  profile_name='UNKNOWN',
                  add_to_known_hosts=False,
                  known_hosts=None,
+                 forward_sshagent=False,
                  terminal_backend=_X2goTerminalSession,
                  info_backend=_X2goServerSessionInfo,
                  list_backend=_X2goServerSessionList,
@@ -140,6 +141,8 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
         @type add_to_known_hosts: C{bool}
         @param known_hosts: the underlying Paramiko/SSH systems C{known_hosts} file
         @type known_hosts: C{str}
+        @param forward_sshagent: forward SSH agent authentication requests to the X2Go client-side
+        @type forward_sshagent: C{bool}
         @param terminal_backend: X2Go terminal session backend to use
         @type terminal_backend: C{class}
         @param info_backend: backend for handling storage of server session information
@@ -163,7 +166,7 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
         @param loglevel: if no L{X2goLogger} object has been supplied a new one will be
             constructed with the given loglevel
         @type loglevel: C{int}
-        @param kwargs: parameters passed through to C{SSHClient.__init__()}
+        @param kwargs: catch any non-defined parameters in C{kwargs}
         @type kwargs: C{dict}
 
         """
@@ -173,6 +176,7 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
         self.profile_name = profile_name
         self.add_to_known_hosts = add_to_known_hosts
         self.known_hosts = known_hosts
+        self.forward_sshagent = forward_sshagent
 
         self.hostname = None
         self.port = None
@@ -204,14 +208,17 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
 
         self._published_applications_menu = {}
 
-        paramiko.SSHClient.__init__(self, **kwargs)
+        self.agent_chan = None
+        self.agent_handler = None
+
+        paramiko.SSHClient.__init__(self)
         if self.add_to_known_hosts:
             self.set_missing_host_key_policy(paramiko.AutoAddPolicy())
 
         self.session_died = False
 
         self.published_applications_no_submenus = published_applications_no_submenus
-        self._already_querying_published_applications = False
+        self._already_querying_published_applications = threading.Lock()
 
         self._transport_lock = threading.Lock()
 
@@ -274,7 +281,7 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
             self.sftp_client = paramiko.SFTPClient.from_transport(ssh_transport)
             try:
                 self.sftp_client.put(os.path.normpath(local_path), remote_path)
-            except x2go_exceptions.SSHException, socket.error:
+            except (x2go_exceptions.SSHException, socket.error, IOError):
                 # react to connection dropped error for SSH connections
                 self.session_died = True
                 self._transport_lock.release()
@@ -306,7 +313,7 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
                 self.logger('sFTP-write: writing content: %s' % content, loglevel=log.loglevel_DEBUG_SFTPXFER)
                 remote_fileobj.write(content)
                 remote_fileobj.close()
-            except x2go_exceptions.SSHException, socket.error:
+            except (x2go_exceptions.SSHException, socket.error, IOError):
                 self.session_died = True
                 self._transport_lock.release()
                 self.logger('sFTP-write: opening remote file %s on host %s failed' % (remote_path, self.remote_peername()), loglevel=log.loglevel_WARN)
@@ -333,7 +340,7 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
             self.sftp_client = paramiko.SFTPClient.from_transport(ssh_transport)
             try:
                 self.sftp_client.remove(remote_path)
-            except x2go_exceptions.SSHException, socket.error:
+            except (x2go_exceptions.SSHException, socket.error, IOError):
                 self.session_died = True
                 self._transport_lock.release()
                 self.logger('sFTP-write: removing remote file %s on host %s failed' % (remote_path, self.remote_peername()), loglevel=log.loglevel_WARN)
@@ -368,6 +375,8 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
         else:
             cmd = cmd_line
 
+        cmd = 'sh -c \"%s\"' % cmd
+
         if self.session_died:
             self.logger("control session seams to be dead, not executing command ,,%s'' on X2Go server %s" % (_rerewrite_blanks(cmd), self.profile_name,), loglevel=loglevel)
             return (cStringIO.StringIO(), cStringIO.StringIO(), cStringIO.StringIO('failed to execute command'))
@@ -438,14 +447,19 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
         else:
             return self._server_features
 
-    def query_server_features(self):
+    def query_server_features(self, force=False):
         """\
         Do a query for the server-side list of X2Go features.
 
+        @param force: do not use the cached feature list, really ask the server (again)
+        @type force: C{bool}
+
         @return: list of X2Go feature names
         @rtype: C{list}
 
         """
+        if force:
+            self._server_features = None
         return self._x2go_server_features
     get_server_features = query_server_features
 
@@ -457,8 +471,10 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
         """
         if self._remote_home is None:
             (stdin, stdout, stderr) = self._x2go_exec_command('echo $HOME')
-            self._remote_home = stdout.read().split()[0]
-            self.logger('remote user\' home directory: %s' % self._remote_home, loglevel=log.loglevel_DEBUG)
+            stdout_r = stdout.read()
+            if stdout_r:
+                self._remote_home = stdout_r.split()[0]
+                self.logger('remote user\' home directory: %s' % self._remote_home, loglevel=log.loglevel_DEBUG)
             return self._remote_home
         else:
             return self._remote_home
@@ -596,9 +612,11 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
         return checkhosts.check_ssh_host_key(self, hostname, port=port)
 
     def connect(self, hostname, port=22, username='', password='', pkey=None,
-                use_sshproxy=False, sshproxy_host='', sshproxy_user='', sshproxy_password='',
-                sshproxy_key_filename='', sshproxy_tunnel='',
                 key_filename=None, timeout=None, allow_agent=False, look_for_keys=False,
+                use_sshproxy=False, sshproxy_host='', sshproxy_port=22, sshproxy_user='', sshproxy_password='', sshproxy_force_password_auth=False,
+                sshproxy_key_filename='', sshproxy_pkey=None, sshproxy_look_for_keys=False, sshproxy_allow_agent=False,
+                sshproxy_tunnel='',
+                forward_sshagent=None,
                 session_instance=None,
                 add_to_known_hosts=False, force_password_auth=False):
         """\
@@ -634,18 +652,22 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
         @param password: a password to use for authentication or for unlocking
             a private key
         @type password: C{str}
-        @param pkey: an optional private key to use for authentication
-        @type pkey: C{PKey}
         @param key_filename: the filename, or list of filenames, of optional
             private key(s) to try for authentication
         @type key_filename: C{str} or list(str)
+        @param pkey: an optional private key to use for authentication
+        @type pkey: C{PKey}
+        @param forward_sshagent: forward SSH agent authentication requests to the X2Go client-side
+            (will update the class property of the same name)
+        @type forward_sshagent: C{bool}
         @param timeout: an optional timeout (in seconds) for the TCP connect
         @type timeout: float
-        @param allow_agent: set to False to disable connecting to the SSH agent
-        @type allow_agent: C{bool}
-        @param look_for_keys: set to False to disable searching for discoverable
+        @param look_for_keys: set to C{True} to enable searching for discoverable
             private key files in C{~/.ssh/}
         @type look_for_keys: C{bool}
+        @param allow_agent: set to C{True} to enable connecting to a local SSH agent
+            for acquiring authentication information
+        @type allow_agent: C{bool}
         @param add_to_known_hosts: non-paramiko option, if C{True} paramiko.AutoAddPolicy() 
             is used as missing-host-key-policy. If set to C{False} paramiko.RejectPolicy() 
             is used
@@ -658,16 +680,27 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
         @type session_instance: C{obj}
         @param use_sshproxy: connect through an SSH proxy
         @type use_sshproxy: C{True} if an SSH proxy is to be used for tunneling the connection
-        @param sshproxy_host: hostname of the SSH proxy server, use <hostname>:<port> to name a
-            non-standard SSH port
+        @param sshproxy_host: hostname of the SSH proxy server
         @type sshproxy_host: C{str}
+        @param sshproxy_port: port of the SSH proxy server
+        @type sshproxy_port: C{int}
         @param sshproxy_user: username that we use for authenticating against C{<sshproxy_host>}
         @type sshproxy_user: C{str}
         @param sshproxy_password: a password to use for SSH proxy authentication or for unlocking
             a private key
         @type sshproxy_password: C{str}
+        @param sshproxy_force_password_auth: enforce using a given C{sshproxy_password} even if a key(file) is given
+        @type sshproxy_force_password_auth: C{bool}
         @param sshproxy_key_filename: local file location of the private key file
         @type sshproxy_key_filename: C{str}
+        @param sshproxy_pkey: an optional private key to use for SSH proxy authentication
+        @type sshproxy_pkey: C{PKey}
+        @param sshproxy_look_for_keys: set to C{True} to enable connecting to a local SSH agent
+            for acquiring authentication information (for SSH proxy authentication)
+        @type sshproxy_look_for_keys: C{bool}
+        @param sshproxy_allow_agent: set to C{True} to enable connecting to a local SSH agent
+            for acquiring authentication information (for SSH proxy authentication)
+        @type sshproxy_allow_agent: C{bool}
         @param sshproxy_tunnel: the SSH proxy tunneling parameters, format is: <local-address>:<local-port>:<remote-address>:<remote-port>
         @type sshproxy_tunnel: C{str}
 
@@ -689,9 +722,14 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
             try:
                 self.sshproxy_session = sshproxy.X2goSSHProxy(known_hosts=self.known_hosts,
                                                               sshproxy_host=sshproxy_host,
+                                                              sshproxy_port=sshproxy_port,
                                                               sshproxy_user=sshproxy_user,
                                                               sshproxy_password=sshproxy_password,
+                                                              sshproxy_force_password_auth=sshproxy_force_password_auth,
                                                               sshproxy_key_filename=sshproxy_key_filename,
+                                                              sshproxy_pkey=sshproxy_pkey,
+                                                              sshproxy_look_for_keys=sshproxy_look_for_keys,
+                                                              sshproxy_allow_agent=sshproxy_allow_agent,
                                                               sshproxy_tunnel=sshproxy_tunnel,
                                                               session_instance=session_instance,
                                                               logger=self.logger,
@@ -734,21 +772,37 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
         if _hostname in ('localhost', 'localhost.localdomain'):
             _hostname = '127.0.0.1'
 
-        if (key_filename and os.path.exists(os.path.normpath(key_filename))) or pkey:
+        # update self.forward_sshagent via connect method parameter
+        if forward_sshagent is not None:
+            self.forward_sshagent = forward_sshagent
+
+        if key_filename or pkey or look_for_keys or allow_agent or (password and force_password_auth):
             try:
-                self.logger('trying SSH pub/priv key authentication with server', loglevel=log.loglevel_DEBUG)
-                if password:
-                    paramiko.SSHClient.connect(self, _hostname, port=port, username=username, pkey=pkey, password=password,
-                                               key_filename=key_filename, timeout=timeout, allow_agent=allow_agent, 
+                if password and force_password_auth:
+                    self.logger('trying keyboard-interactive SSH authentication with server', loglevel=log.loglevel_DEBUG)
+                    paramiko.SSHClient.connect(self, _hostname, port=port, username=username, pkey=None, password=password,
+                                               key_filename=None, timeout=timeout, allow_agent=False,
+                                               look_for_keys=False)
+                elif (key_filename and os.path.exists(os.path.normpath(key_filename))) or pkey:
+                    self.logger('trying SSH pub/priv key authentication with server', loglevel=log.loglevel_DEBUG)
+                    paramiko.SSHClient.connect(self, _hostname, port=port, username=username, pkey=pkey,
+                                               key_filename=key_filename, timeout=timeout, allow_agent=allow_agent,
                                                look_for_keys=look_for_keys)
                 else:
-                    paramiko.SSHClient.connect(self, _hostname, port=port, username=username, pkey=pkey,
-                                               key_filename=key_filename, timeout=timeout, allow_agent=allow_agent, 
-                                               look_for_keys=look_for_keys)
+                    self.logger('trying SSH key discovery or agent authentication with server', loglevel=log.loglevel_DEBUG)
+                    try:
+                        paramiko.SSHClient.connect(self, _hostname, port=port, username=username, pkey=None,
+                                                      key_filename=None, timeout=timeout, allow_agent=allow_agent,
+                                                      look_for_keys=look_for_keys)
+                    except paramiko.SSHException, e:
+                        if str(e) == 'No authentication methods available':
+                            raise paramiko.AuthenticationException('Interactive password authentication required!')
+                        else:
+                            raise(e)
 
                 # since Paramiko 1.7.7.1 there is compression available, let's use it if present...
                 t = self.get_transport()
-                if hasattr(t, 'use_compression'):
+                if x2go._paramiko.PARAMIKO_FEATURE['use-compression']:
                     t.use_compression(compress=True)
 
             except paramiko.AuthenticationException, e:
@@ -757,13 +811,7 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
                     self.logger('next auth mechanism we\'ll try is keyboard-interactive authentication', loglevel=log.loglevel_DEBUG)
                     try:
                         paramiko.SSHClient.connect(self, _hostname, port=port, username=username, password=password,
-                                                   timeout=timeout, allow_agent=allow_agent, 
-                                                   look_for_keys=look_for_keys)
-                    except paramiko.AuthenticationException, e:
-                        self.close()
-                        if self.sshproxy_session:
-                            self.sshproxy_session.stop_thread()
-                        raise e
+                                                   key_filename=None, pkey=None, timeout=timeout, allow_agent=False, look_for_keys=False)
                     except:
                         self.close()
                         if self.sshproxy_session:
@@ -781,7 +829,7 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
                     self.sshproxy_session.stop_thread()
                 raise
 
-        # if there is not private key, we will use the given password, if any
+        # if there is no private key (and no agent auth), we will use the given password, if any
         else:
             # create a random password if password is empty to trigger host key validity check
             if not password:
@@ -789,7 +837,7 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
             self.logger('performing SSH keyboard-interactive authentication with server', loglevel=log.loglevel_DEBUG)
             try:
                 paramiko.SSHClient.connect(self, _hostname, port=port, username=username, password=password,
-                                           timeout=timeout, allow_agent=allow_agent, look_for_keys=look_for_keys)
+                                           timeout=timeout, allow_agent=False, look_for_keys=False)
             except paramiko.AuthenticationException, e:
                 self.close()
                 if self.sshproxy_session:
@@ -814,9 +862,18 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
         ssh_transport._x2go_session_marker = True
         self._session_password = password
 
-        if self.get_transport():
+        if ssh_transport is not None:
             self.session_died = False
+            self.query_server_features(force=True)
+            if self.forward_sshagent:
+                if x2go._paramiko.PARAMIKO_FEATURE['forward-ssh-agent']:
+                    self.agent_chan = ssh_transport.open_session()
+                    self.agent_handler = paramiko.agent.AgentRequestHandler(self.agent_chan)
+                    self.logger('Requesting SSH agent forwarding for control session of connected session profile %s' % self.profile_name, loglevel=log.loglevel_INFO)
+                else:
+                    self.logger('SSH agent forwarding is not available in the Paramiko version used with this instance of Python X2Go', loglevel=log.loglevel_WARN)
 
+        self._remote_home = None
         if not self.home_exists():
             raise x2go_exceptions.X2goRemoteHomeException('remote home directory does not exist')
 
@@ -869,21 +926,31 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
         # in any case, release out internal transport lock
         self._transport_lock.release()
 
+        # close SSH agent auth forwarding objects
+        if self.agent_handler is not None:
+            self.agent_handler.close()
+
+        if self.agent_chan is not None:
+            self.agent_chan.close()
+
+        retval = False
         try:
             if self.get_transport() is not None:
-                still_active = self.get_transport().is_active()
+                retval = self.get_transport().is_active()
                 try:
                     self.close()
                 except IOError:
                     pass
-                if self.sshproxy_session is not None:
-                    self.sshproxy_session.stop_thread()
-                return still_active
-            return False
         except AttributeError:
             # if the Paramiko _transport object has not yet been initialized, ignore it
             # but state that this method call did not close the SSH client, but was already closed
-            return False
+            pass
+
+        # take down sshproxy_session no matter what happened to the control session itself
+        if self.sshproxy_session is not None:
+            self.sshproxy_session.stop_thread()
+
+        return retval
 
     def home_exists(self):
         """\
@@ -947,19 +1014,14 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
         @rtype: C{list}
 
         """
-
-        if self._already_querying_published_applications:
-            self.logger('This control session instance is currently already querying the published applications menu tree for session profile %s. Whenever this warning pops up in your log file it means that you should fix your client implementation code. Only call this method once per session profile!!!' % self.profile_name, loglevel=log.loglevel_WARN)
-            return None
-        else:
-            self._already_querying_published_applications = True
+        self._already_querying_published_applications.acquire()
 
         if defaults.X2GOCLIENT_OS != 'Windows' and lang is None:
             lang = locale.getdefaultlocale()[0]
         elif lang is None:
             lang = 'en'
 
-        if 'X2GO_PUBLISHED_APPLICATIONS' in self._x2go_server_features:
+        if 'X2GO_PUBLISHED_APPLICATIONS' in self.get_server_features():
             if self._published_applications_menu is {} or \
                not self._published_applications_menu.has_key(lang) or \
                raw or very_raw or refresh or \
@@ -975,6 +1037,7 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
 
                 if very_raw:
                     self.logger('published applications query for %s finished, return very raw output' % self.profile_name, loglevel=log.loglevel_NOTICE)
+                    self._already_querying_published_applications.release()
                     return _raw_output
 
                 ### STAGE 2: dissect the text file retrieved from server, cut into single menu elements
@@ -996,6 +1059,7 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
 
                 if raw:
                     self.logger('published applications query for %s finished, returning raw output' % self.profile_name, loglevel=log.loglevel_NOTICE)
+                    self._already_querying_published_applications.release()
                     return _menu
 
                 if len(_menu) > max_no_submenus >= 0:
@@ -1069,7 +1133,7 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
                                 continue
                             elif 'System' in line:
                                 _menu_entry_cat = 'System'
-                            elif 'Utilities' in line:
+                            elif 'Utility' in line:
                                 _menu_entry_cat = 'Utilities'
                             else:
                                 _menu_entry_cat = 'Other Applications'
@@ -1120,10 +1184,9 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
             # FIXME: ignoring the absence of the published applications feature for now, handle it appropriately later
             pass
 
-        self._already_querying_published_applications = False
+        self._already_querying_published_applications.release()
         return self._published_applications_menu
 
-
     def start(self, **kwargs):
         """\
         Start a new X2Go session.
@@ -1162,13 +1225,16 @@ class X2goControlSessionSTDOUT(paramiko.SSHClient):
         if not self.is_x2gouser(self.get_transport().get_username()):
             raise x2go_exceptions.X2goUserException('remote user %s is not allowed to run X2Go commands' % self.get_transport().get_username())
 
-        if session_name is not None:
-            if session_list:
-                session_info = session_list[session_name]
+        try:
+            if session_name is not None:
+                if session_list:
+                    session_info = session_list[session_name]
+                else:
+                    session_info = self.list_sessions()[session_name]
             else:
-                session_info = self.list_sessions()[session_name]
-        else:
-            session_info = None
+                session_info = None
+        except KeyError:
+            _success = False
 
         _terminal = self._terminal_backend(self,
                                            profile_name=self.profile_name,
