@@ -28,6 +28,7 @@ __NAME__ = 'x2gosessionprofiles-pylib'
 
 import copy
 import types
+import re
 
 # Python X2Go modules
 from x2go.defaults import X2GO_SESSIONPROFILES_CONFIGFILES as _X2GO_SESSIONPROFILES_CONFIGFILES
@@ -177,12 +178,14 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
         # we have to handle the get_type method separately...
         return self.get_profile_option_type(key)
 
-    def get_profile_config(self, profile_id_or_name=None, profile_id=None):
+    def get_profile_config(self, profile_id_or_name=None, parameter=None, profile_id=None):
         """\
         The configuration options for a single session profile.
 
         @param profile_id_or_name: either profile ID or profile name is accepted
         @type profile_id_or_name: C{str}
+        @param parameter: if specified, only the value for the given parameter is returned
+        @type parameter: C{str}
         @param profile_id: profile ID (fast than specifying C{profile_id_or_name})
         @type profile_id: C{str}
 
@@ -194,6 +197,27 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
         _profile_config = {}
         for option in self.iniConfig.options(_profile_id):
             _profile_config[option] = self.get(_profile_id, option, key_type=self.get_profile_option_type(option))
+
+        if parameter is not None:
+            if parameter in _profile_config.keys():
+
+                value = _profile_config[parameter]
+
+                if parameter == 'export':
+                    _strvalue = value.replace(',', ';').strip().strip('"').strip().strip(';').strip()
+                    value = {}
+                    if _strvalue:
+                        _export_paths = _strvalue.split(';')
+                        for _path in _export_paths:
+                            if not re.match('.*:(0|1)$', _path): _path = '%s:1' % _path
+                            _auto_export_path = re.match('.*:1$', _path) and True or False
+                            _export_path = ':'.join(_path.split(':')[:-1])
+                            value[_export_path] = _auto_export_path
+
+                return value
+
+            else:
+                raise X2goProfileException('no such session profile parameter: %s' % parameter)
         return _profile_config
 
     def default_profile_config(self):
@@ -362,17 +386,30 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
         @type value: any type, depends on the session profile option
 
         """
-        profile_id = section
+        try:
+            profile_id = self.check_profile_id_or_name(section)
+        except X2goProfileException:
+            profile_id = section
         if key == 'name':
             profile_name = value
-            current_profile_name = self.get_value(section, key)
+            current_profile_name = self.get_value(profile_id, key)
             if not profile_name:
                 raise X2goProfileException('profile name for profile id %s may not be empty' % profile_id)
             else:
                 if profile_name != current_profile_name and profile_name in self.profile_names:
                     raise X2goProfileException('a profile of name ,,%s'' already exists' % profile_name)
             self._cached_profile_names = []
-        inifiles.X2goIniFile.update_value(self, section, key, value)
+
+        if key == 'export' and type(value) == types.DictType:
+
+            _strvalue = '"'
+            for folder in value.keys():
+                _strvalue += "%s:%s;" % (folder, int(value[folder]))
+            _strvalue += '"'
+            _strvalue = _strvalue.replace('""', '')
+            value = _strvalue
+
+        inifiles.X2goIniFile.update_value(self, profile_id, key, value)
 
     def check_profile_id_or_name(self, profile_id_or_name):
         """\
