@@ -105,6 +105,7 @@ class X2goProxyBASE(threading.Thread):
 
         self.sessions_rootdir = sessions_rootdir
         self.session_info = session_info
+        self.session_name = self.session_info.name
         self.ssh_transport = ssh_transport
         self.session_log = session_log
         self.proxy_options = proxy_options
@@ -186,6 +187,7 @@ class X2goProxyBASE(threading.Thread):
                                                       remote_port=self.session_info.graphics_port,
                                                       ssh_transport=self.ssh_transport,
                                                       session_instance=self.session_instance,
+                                                      session_name=self.session_name,
                                                       logger=self.logger,
                                                      )
 
