@@ -143,10 +143,9 @@ class X2goSessionParams(object):
             self.session_type = 'P'
             self.cmd = 'PUBLISHED'
         else:
-            if cmd == 'RDP':
-                self.session_type = 'R'
-            elif cmd.startswith('rdesktop'):
-                self.session_type = 'R'
+            if cmd == 'RDP' or cmd.startswith('rdesktop') or cmd.startswith('xfreedrp'):
+                if self.geometry == 'fullscreen': self.session_type = 'D'
+                else: self.session_type = 'R'
             elif cmd == 'XDMCP':
                 self.session_type = 'D'
             elif cmd in _X2GO_DESKTOPSESSIONS.keys():
@@ -228,8 +227,8 @@ class X2goTerminalSessionSTDOUT(object):
         new X2Go sessions, resume suspended sessions or suspend resp. terminate
         currently running sessions on a connected X2Go server.
 
-        @param geometry: screen geometry of the X2Go session. Can be either C{<width>x<height>}
-            or C{fullscreen}
+        @param geometry: screen geometry of the X2Go session. Can be either C{<width>x<height>},
+            C{maximize} or C{fullscreen}
         @type geometry: C{str}
         @param depth: color depth in bits (common values: C{16}, C{24})
         @type depth: C{int}
@@ -845,17 +844,17 @@ class X2goTerminalSessionSTDOUT(object):
                 _client_encoding = 'WINDOWS-1252'
 
             if _convert_encoding:
-                export_iconv_settings = 'export X2GO_ICONV=modules=iconv,from_code=%s,to_code=%s &&' % (_client_encoding, _server_encoding)
+                export_iconv_settings = 'export X2GO_ICONV=modules=iconv,from_code=%s,to_code=%s && ' % (_client_encoding, _server_encoding)
             else:
                 export_iconv_settings = ''
 
             if folder_type == 'disk':
 
-                cmd_line = [ '%s export HOSTNAME &&' % export_iconv_settings,
+                cmd_line = [ '%sexport HOSTNAME &&' % export_iconv_settings,
                              'x2gomountdirs',
                              'dir',
                              str(self.session_info.name),
-                             '"%s"' % _CURRENT_LOCAL_USER,
+                             '\'%s\'' % _CURRENT_LOCAL_USER,
                              _x2go_key_fname,
                              '%s__REVERSESSH_PORT__%s; ' % (local_path, self.session_info.sshfs_port),
                              'rm -f %s %s.ident' % (_x2go_key_fname, _x2go_key_fname),
@@ -863,11 +862,11 @@ class X2goTerminalSessionSTDOUT(object):
 
             elif folder_type == 'spool':
 
-                cmd_line = [ '%s export HOSTNAME &&' % export_iconv_settings,
+                cmd_line = [ '%sexport HOSTNAME &&' % export_iconv_settings,
                              'x2gomountdirs',
                              'dir',
                              str(self.session_info.name),
-                             '"%s"' % _CURRENT_LOCAL_USER,
+                             '\'%s\'' % _CURRENT_LOCAL_USER,
                              _x2go_key_fname,
                              '%s__PRINT_SPOOL___REVERSESSH_PORT__%s; ' % (local_path, self.session_info.sshfs_port),
                              'rm -f %s %s.ident' % (_x2go_key_fname, _x2go_key_fname), 
@@ -875,11 +874,11 @@ class X2goTerminalSessionSTDOUT(object):
 
             elif folder_type == 'mimebox':
 
-                cmd_line = [ '%s export HOSTNAME &&' % export_iconv_settings,
+                cmd_line = [ '%sexport HOSTNAME &&' % export_iconv_settings,
                              'x2gomountdirs',
                              'dir',
                              str(self.session_info.name),
-                             '"%s"' % _CURRENT_LOCAL_USER,
+                             '\'%s\'' % _CURRENT_LOCAL_USER,
                              _x2go_key_fname,
                              '%s__MIMEBOX_SPOOL___REVERSESSH_PORT__%s; ' % (local_path, self.session_info.sshfs_port),
                              'rm -f %s %s.ident' % (_x2go_key_fname, _x2go_key_fname), 
@@ -988,7 +987,7 @@ class X2goTerminalSessionSTDOUT(object):
         if self.session_title != _generic_title and not dont_set:
             self.set_session_window_title(title=self.session_title)
 
-    def find_session_window(self, timeout=30):
+    def find_session_window(self, timeout=60):
         """\
         Try for <timeout> seconds to find the X2Go session window of this
         terminal session.
@@ -1027,7 +1026,7 @@ class X2goTerminalSessionSTDOUT(object):
 
             gevent.sleep(1)
 
-    def set_session_window_title(self, title, timeout=30):
+    def set_session_window_title(self, title, timeout=60):
         """\
         Modify the session window title.
 
@@ -1067,7 +1066,7 @@ class X2goTerminalSessionSTDOUT(object):
 
             gevent.sleep(1)
 
-    def raise_session_window(self, timeout=30):
+    def raise_session_window(self, timeout=60):
         """\
         Try for <timeout> seconds to raise the X2Go session window of this
         terminal session to the top and bring it to focus.
@@ -1160,7 +1159,7 @@ class X2goTerminalSessionSTDOUT(object):
         @rtype: C{tuple} of C{str}
 
         """
-        if not self.has_command(_rewrite_cmd(self.params.cmd, params=self.params)):
+        if not self.has_command(_rewrite_cmd(str(self.params.cmd), params=self.params)):
             if self.client_instance:
                 self.client_instance.HOOK_no_such_command(profile_name=self.profile_name, session_name=self.session_info.name, cmd=self.params.cmd)
             return False
@@ -1193,7 +1192,7 @@ class X2goTerminalSessionSTDOUT(object):
                      _rewrite_blanks(_rewrite_cmd(cmd, params=self.params)),
                      str(self.params.snd_system),
                      str(self.params.session_type),
-                     "&> /dev/null & exit",
+                     "1>/dev/null 2>/dev/null & exit",
                    ]
 
         if self.params.snd_system == 'pulse':
@@ -1297,7 +1296,7 @@ class X2goTerminalSessionSTDOUT(object):
         cmd_line.extend(
             [
                 "setsid %s" % exec_name,
-                "&> /dev/null & exit",
+                "1>/dev/null 2>/dev/null & exit",
             ]
         )
 
@@ -1375,6 +1374,16 @@ class X2goTerminalSessionSTDOUT(object):
 
         self.params.rewrite_session_type()
 
+        if self.params.geometry == 'maximize':
+            _geometry = utils.get_workarea_geometry()
+            if _geometry is None or len(_geometry) != 2:
+                _geometry = utils.get_desktop_geometry()
+            if _geometry and len(_geometry) == 2:
+                self.params.geometry = "%sx%s" % _geometry
+            else:
+                self.logger('failed to detect best maxmimized geometry of your client-side desktop', loglevel=log.loglevel_WARN)
+                self.params.geometry = "1024x768"
+
         cmd_line = [ "x2gostartagent",
                      str(self.params.geometry),
                      str(self.params.link),
@@ -1384,7 +1393,7 @@ class X2goTerminalSessionSTDOUT(object):
                      str(self.params.kbtype),
                      str(setkbd),
                      str(self.params.session_type),
-                     self.params.cmd,
+                     str(self.params.cmd),
                    ]
 
         if self.params.cmd == 'XDMCP' and self.params.xdmcp_server:
@@ -1459,6 +1468,16 @@ class X2goTerminalSessionSTDOUT(object):
         if self.params.kbtype != "null/null":
             setkbd = "1"
 
+        if self.params.geometry == 'maximize':
+            _geometry = utils.get_workarea_geometry()
+            if _geometry is None or len(_geometry) != 2:
+                _geometry = utils.get_desktop_geometry()
+            if _geometry and len(_geometry) == 2:
+                self.params.geometry = "%sx%s" % _geometry
+            else:
+                self.logger('failed to detect best maxmimized geometry of your client-side desktop, using 1024x768 instead', loglevel=log.loglevel_WARN)
+                self.params.geometry = "1024x768"
+
         cmd_line = [ "x2goresume-session", self.session_info.name,
                      self.params.geometry,
                      self.params.link,
