@@ -1,18 +1,18 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2011 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2012 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
-# Python X2go is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
+# Python X2Go is free software; you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 #
-# Python X2go is distributed in the hope that it will be useful,
+# Python X2Go is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program; if not, write to the
 # Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
@@ -26,16 +26,17 @@ __NAME__ = 'x2gocheckhosts-pylib'
 # modules
 import paramiko
 import binascii
-import uuid
 
-# Python X2go modules
+# Python X2Go modules
 import sshproxy
 import log
 import x2go_exceptions
+import random
+import string
 
 class X2goInteractiveAddPolicy(paramiko.MissingHostKeyPolicy):
     """\
-    Policy for making host key information available to Python X2go after a
+    Policy for making host key information available to Python X2Go after a
     Paramiko/SSH connect has been attempted. This class needs information
     about the associated L{X2goSession} instance.
 
@@ -53,7 +54,7 @@ class X2goInteractiveAddPolicy(paramiko.MissingHostKeyPolicy):
         """\
         @param caller: calling instance
         @type caller: C{class}
-        @param session_instance: an X2go session instance
+        @param session_instance: an X2Go session instance
         @type session_instance: L{X2goSession} instance
 
         """
@@ -80,6 +81,10 @@ class X2goInteractiveAddPolicy(paramiko.MissingHostKeyPolicy):
         @param key: host key to validate
         @type key: Paramiko/SSH key instance
 
+        @raise X2goHostKeyException: if the X2Go server host key is not in the C{known_hosts} file
+        @raise X2goSSHProxyHostKeyException: if the SSH proxy host key is not in the C{known_hosts} file
+        @raise SSHException: if this instance does not know its {self.session_instance}
+
         """
         self.client = client
         self.hostname = hostname
@@ -90,7 +95,7 @@ class X2goInteractiveAddPolicy(paramiko.MissingHostKeyPolicy):
         client._log(paramiko.common.DEBUG, 'Interactively Checking %s host key for %s: %s' %
                    (self.key.get_name(), self.hostname, binascii.hexlify(self.key.get_fingerprint())))
         if self.session_instance:
-            self.session_instance.logger('SSH host key verification for host %s with %s fingerprint ,,%s\'\' initiated. We are seeing this X2go server for the first time.' % (self.get_hostname(), self.get_key_name(), self.get_key_fingerprint_with_colons()), loglevel=log.loglevel_NOTICE)
+            self.session_instance.logger('SSH host key verification for host %s with %s fingerprint ,,%s\'\' initiated. We are seeing this X2Go server for the first time.' % (self.get_hostname(), self.get_key_name(), self.get_key_fingerprint_with_colons()), loglevel=log.loglevel_NOTICE)
             _valid = self.session_instance.HOOK_check_host_dialog(self.get_hostname_name(),
                                                                   port=self.get_hostname_port(),
                                                                   fingerprint=self.get_key_fingerprint_with_colons(),
@@ -110,7 +115,7 @@ class X2goInteractiveAddPolicy(paramiko.MissingHostKeyPolicy):
         """\
         Retrieve the Paramiko SSH/Client.
 
-        @return: the associated X2go control session instance.
+        @return: the associated X2Go control session instance.
         @rtype: C{X2goControlSession*} instance
 
         """
@@ -208,9 +213,12 @@ def check_ssh_host_key(x2go_sshclient_instance, hostname, port=22):
     @type hostname: C{str}
     @param port: port of server to validate
     @type port: C{int}
+
     @return: returns a tuple with the following components (<host_ok>, <hostname>, <port>, <fingerprint>, <fingerprint_type>)
     @rtype: C{tuple}
 
+    @raise SSHException: if an SSH exception occurred, that we did not provocate in L{X2goInteractiveAddPolicy.missing_host_key()}
+
     """
     _hostname = hostname
     _port = port
