@@ -41,7 +41,7 @@ if X2GOCLIENT_OS != 'Windows':
     try:
         X_DISPLAY = Xlib.display.Display()
     except Xlib.error.DisplayNameError:
-	X_DISPLAY = None
+        X_DISPLAY = None
     except Xlib.error.DisplayConnectionError:
         X_DISPLAY = None
 
@@ -179,33 +179,33 @@ X2GO_CLIENTSETTINGS_DEFAULTS = {
         'server': 'localhost',
         'port1': 0,
         'port2': 0,
-        },
+    },
     'General': {
         # clientport is not needed for Python X2go
         'clientport': 22, 
         'autoresume': True,
-        },
+    },
     'Authorization': {
         'newprofile': True,
         'suspend': True,
         'editprofile': True,
         'resume': True
-        },
+    },
     'trayicon': {
         'enabled': True,
         'mintotray': True,
         'noclose': True,
         'mincon': True,
         'maxdiscon': True,
-        },
-    }
+    },
+}
 X2GO_CLIENTPRINTING_DEFAULTS = {
     'General': {
         # showdialog will result in a print action that allows opening a print dialog box
         'showdialog': False,
         # if true, open a PDF viewer (or save as PDF file). If false, print via CUPS or print command
         'pdfview': True,
-        },
+    },
     'print': {
         # If false, print via CUPS. If true, run "command" to process the print job
         'startcmd': False,
@@ -215,7 +215,7 @@ X2GO_CLIENTPRINTING_DEFAULTS = {
         'stdin': False,
         # ignored in Python X2go
         'ps': False,
-        },
+    },
     'save': {
         # a path relative to the user's home directory
         'folder': 'PDF',
@@ -227,13 +227,13 @@ X2GO_CLIENTPRINTING_DEFAULTS = {
         'open': True,
         # command to execute as PDF viewer
         'command': 'xpdf',
-        },
+    },
     'CUPS': {
         # default print queue for CUPS, if print queue does not exist, the default 
         # CUPS queue is detected
         'defaultprinter': 'PDF',
-        },
-    }
+    },
+}
 if X2GOCLIENT_OS == 'Windows':
     X2GO_CLIENTPRINTING_DEFAULTS['print'].update({'gsprint': os.path.join(os.environ['ProgramFiles'], 'GhostGum', 'gsview', 'gsprint.exe'), })
 
@@ -250,7 +250,7 @@ if X2GOCLIENT_OS == 'Windows':
             'test_installed': os.path.join(os.environ['SystemDrive'], '\\', 'cygwin', 'bin', 'XWin.exe'),
             'run_command': os.path.join(os.environ['SystemDrive'], '\\', 'cygwin', 'bin', 'XWin.exe'),
             'parameters': [':40', '-clipboard', '-multiwindow', '-notrayicon', '-nowinkill', '-nounixkill', '-swcursor', ],
-            },
+        },
         'VcXsrv': {
             'display': 'localhost:40',
             'last_display': 'localhost:40',
@@ -258,7 +258,7 @@ if X2GOCLIENT_OS == 'Windows':
             'test_installed': os.path.join(os.environ['ProgramFiles'], 'VcXsrv', 'vcxsrv.exe'), 
             'run_command': os.path.join(os.environ['ProgramFiles'], 'VcXsrv', 'vcxsrv.exe'),
             'parameters': [':40', '-clipboard', '-multiwindow', '-notrayicon', '-nowinkill', '-nounixkill', '-swcursor', ],
-            },
+        },
         'VcXsrv_shipped': {
             'display': 'localhost:40',
             'last_display': 'localhost:40',
@@ -266,7 +266,7 @@ if X2GOCLIENT_OS == 'Windows':
             'test_installed': os.path.join(os.getcwd(), 'VcXsrv', 'vcxsrv.exe'), 
             'run_command': os.path.join(os.getcwd(), 'VcXsrv', 'vcxsrv.exe'),
             'parameters': [':40', '-clipboard', '-multiwindow', '-notrayicon', '-nowinkill', '-nounixkill', '-swcursor', ],
-            },
+        },
         'VcXsrv_development': {
             'display': 'localhost:40',
             'last_display': 'localhost:40',
@@ -274,7 +274,7 @@ if X2GOCLIENT_OS == 'Windows':
             'test_installed': os.path.join(os.getcwd(), '..', 'pyhoca-contrib', 'mswin', 'vcxsrv-mswin', 'VcXsrv-1.12.0.1', 'vcxsrv.exe'), 
             'run_command': os.path.join(os.getcwd(), '..', 'pyhoca-contrib', 'mswin', 'vcxsrv-mswin', 'VcXsrv-1.12.0.1', 'vcxsrv.exe'), 
             'parameters': [':40', '-clipboard', '-multiwindow', '-notrayicon', '-nowinkill', '-nounixkill', '-swcursor', ],
-            },
+        },
         'Xming': {
             'display': 'localhost:40',
             'last_display': 'localhost:40',
@@ -282,8 +282,8 @@ if X2GOCLIENT_OS == 'Windows':
             'test_installed': os.path.join(os.environ['ProgramFiles'], 'Xming', 'Xming.exe'), 
             'run_command': os.path.join(os.environ['ProgramFiles'], 'Xming', 'Xming.exe'),
             'parameters': [':40', '-clipboard', '-multiwindow', '-notrayicon', '-nowinkill', '-nounixkill', '-swcursor', ],
-            },
-        }
+        },
+    }
 else:
     # make the variable available when building API documentation with epydoc
     X2GO_CLIENTXCONFIG_DEFAULTS = {}
@@ -292,23 +292,25 @@ X2GO_GENERIC_APPLICATIONS = [ 'WWWBROWSER', 'MAILCLIENT', 'OFFICE', 'TERMINAL',
 """X2go's generic applications."""
 
 X2GO_SESSIONPROFILE_DEFAULTS = {
-    'autologin': False, 'autostart': False, 'setsessiontitle': False, 'sessiontitle': "",
+    'autologin': True, 'autoconnect': False, 'autostart': False, 'setsessiontitle': False, 'sessiontitle': "",
     'speed': 2, 'pack': '16m-jpeg', 'quality': 9,
     'iconvto': 'UTF-8', 'iconvfrom': 'UTF-8', 'useiconv': False,
-    'usesshproxy': False, 'sshproxyhost': '', 'sshproxyuser': '', 'sshproxytunnel': '', 'sshproxykeyfile': '',
-    'useexports': True, 'fstunnel': True, 'export': '',
+    'usesshproxy': False, 'sshproxyhost': 'proxyhost.mydomain', 'sshproxyport': 22, 'sshproxyuser': '', 'sshproxytunnel': 'localhost:44444:server.mydomain.private:22', 'sshproxykeyfile': '',
+    'sshproxytype': 'SSH', 'sshproxysameuser': False, 'sshproxysamepass': False, 'sshproxyautologin': True,
+    'useexports': True, 'restoreexports': False, 'fstunnel': True, 'export': '',
     'usemimebox': False, 'mimeboxextensions': '', 'mimeboxaction': 'OPEN',
     'fullscreen': False,
-    'width': 800,'height': 600,'dpi': 96,'setdpi': False, 'xinerama': False, 'multidisp': False,
+    'width': 800,'height': 600, 'maxdim': False, 'dpi': 96, 'setdpi': False, 'xinerama': False, 'multidisp': False,
     'usekbd': True, 'layout': 'us', 'type': 'pc105/us', 'variant': '',
     'sound': False, 'soundsystem': 'pulse', 'startsoundsystem': False, 'soundtunnel':True, 'defsndport':True, 'sndport':4713,
     'name': 'NEW_PROFILE', 'icon': ':icons/128x128/x2gosession.png',
-    'host': '', 'user': CURRENT_LOCAL_USER, 'key': '', 'sshport': 22, 'krblogin': False,
+    'host': 'server.mydomain', 'user': CURRENT_LOCAL_USER, 'key': '', 'sshport': 22, 'krblogin': False, 'forwardsshagent': False,
     'rootless': True, 'applications': X2GO_GENERIC_APPLICATIONS, 'command':'TERMINAL', 'published': False,
+    'directrdp': False, 'directrdpsettings': '', 'rdpclient': 'rdesktop', 'rdpport': 3389,
     'rdpoptions': '-u X2GO_USER -p X2GO_PASSWORD', 'rdpserver': '',
     'print': False,
     'xdmcpserver': 'localhost',
-    }
+}
 """L{X2goSessionProfiles} default values to fill a new session profile with."""
 ##
 ## X2Go Proxy defaults
@@ -374,7 +376,7 @@ pack_methods_nx3.sort()
 X2GO_DESKTOPSESSIONS={
     'KDE': 'startkde',
     'GNOME': 'gnome-session',
-    'XFCE4': 'xfce4-session',
+    'XFCE': 'xfce4-session',
     'LXDE': 'startlxde',
     'TRINITY': 'starttrinity',
     'UNITY': 'unity',
