@@ -45,7 +45,7 @@ class X2goFwServer(StreamServer):
     through an external proxy command launched by a C{X2goProxy*} backend.
 
     """
-    def __init__ (self, listener, remote_host, remote_port, ssh_transport, session_instance=None, logger=None, loglevel=log.loglevel_DEFAULT,):
+    def __init__ (self, listener, remote_host, remote_port, ssh_transport, session_instance=None, session_name=None, logger=None, loglevel=log.loglevel_DEFAULT,):
         """\
         @param listener: listen on TCP/IP socket C{(<IP>, <Port>)}
         @type listener: C{tuple}
@@ -55,6 +55,11 @@ class X2goFwServer(StreamServer):
         @type remote_port: C{int}
         @param ssh_transport: a valid Paramiko/SSH transport object
         @type ssh_transport: C{obj}
+        @param session_instance: the complete L{X2goSession} instance of the X2Go session this port forwarding server belongs to.
+            Note: for new L{X2goSession} instances the object has the session name not yet set(!!!)
+        @type session_instance: C{obj}
+        @param session_name: the session name of the X2Go session this port forwarding server belongs to
+        @type session_name: C{str}
         @param logger: you can pass an L{X2goLogger} object to the
             L{X2goFwServer} constructor
         @type logger: C{obj}
@@ -76,6 +81,7 @@ class X2goFwServer(StreamServer):
         self.chain_host = remote_host
         self.chain_port = remote_port
         self.ssh_transport = ssh_transport
+        self.session_name = session_name
         self.session_instance = session_instance
 
         self.fw_socket = None
@@ -130,6 +136,7 @@ class X2goFwServer(StreamServer):
                                                                                 _count),
                                                                                 loglevel=log.loglevel_ERROR)
             if self.session_instance:
+                self.session_instance.set_session_name(self.session_name)
                 self.session_instance.HOOK_forwarding_tunnel_setup_failed(chain_host=self.chain_host, chain_port=self.chain_port)
             self.failed = True
 
@@ -216,6 +223,7 @@ def start_forward_tunnel(local_host='127.0.0.1', local_port=22022,
                          remote_host='127.0.0.1', remote_port=22,
                          ssh_transport=None, 
                          session_instance=None,
+                         session_name=None,
                          logger=None, ):
     """\
     Setup up a Paramiko/SSH port forwarding tunnel (like openssh -L option).
@@ -230,10 +238,12 @@ def start_forward_tunnel(local_host='127.0.0.1', local_port=22022,
     @type remote_host: C{str}
     @param remote_port: ... on port C{<remote_port>}
     @type remote_port: C{int}
-    @param ssh_transport: the Paramiko/SSH transport (i.e. the X2Go sessions Paramiko/SSH transport object)
+    @param ssh_transport: the Paramiko/SSH transport (i.e. the X2Go session's Paramiko/SSH transport object)
     @type ssh_transport: C{obj}
     @param session_instance: the L{X2goSession} instance that initiates this tunnel
     @type session_instance: C{obj}
+    @param session_name: the session name of the X2Go session this port forwarding server belongs to
+    @type session_name: C{str}
     @param logger: an X2goLogger object
     @type logger: C{obj}
 
@@ -243,7 +253,8 @@ def start_forward_tunnel(local_host='127.0.0.1', local_port=22022,
     """
     fw_server = X2goFwServer(listener=(local_host, local_port),
                              remote_host=remote_host, remote_port=remote_port,
-                             ssh_transport=ssh_transport, session_instance=session_instance,
+                             ssh_transport=ssh_transport,
+                             session_instance=session_instance, session_name=session_name,
                              logger=logger,
                             )
     try:
