@@ -50,6 +50,7 @@ class X2goSessionGuardian(threading.Thread):
                  auto_update_listmounts_cache=False, 
                  auto_update_sessionregistry=False,
                  auto_register_sessions=False,
+                 no_auto_reg_pubapp_sessions=False,
                  refresh_interval=5,
                  logger=None, loglevel=log.loglevel_DEFAULT):
         """\
@@ -64,6 +65,8 @@ class X2goSessionGuardian(threading.Thread):
         @param auto_register_sessions: register new sessions automatically once they appear in the X2Go session (e.g. 
             instantiated by another client that is connected to the same X2Go server under same user ID)
         @type auto_register_sessions: C{bool}
+        @param no_auto_reg_pubapp_sessions: do not auto-register published applications sessions
+        @type no_auto_reg_pubapp_sessions: C{bool}
         @param refresh_interval: refresh cache and session registry every <refresh_interval> seconds
         @type refresh_interval: C{int}
         @param logger: you can pass an L{X2goLogger} object to the L{X2goSessionGuardian} constructor
@@ -85,6 +88,7 @@ class X2goSessionGuardian(threading.Thread):
         self.auto_update_listmounts_cache = auto_update_listmounts_cache
         self.auto_update_sessionregistry = auto_update_sessionregistry
         self.auto_register_sessions = auto_register_sessions
+        self.no_auto_reg_pubapp_sessions = no_auto_reg_pubapp_sessions
         self.refresh_interval = refresh_interval
 
         threading.Thread.__init__(self, target=self.guardian)
@@ -117,7 +121,7 @@ class X2goSessionGuardian(threading.Thread):
 
                 # session auto-registration will automatically trigger an update of the session registry status
                 if self.auto_register_sessions:
-                    self.client_instance.register_available_server_sessions_all_profiles()
+                    self.client_instance.register_available_server_sessions_all_profiles(skip_pubapp_sessions=self.no_auto_reg_pubapp_sessions)
 
         self.logger('X2Go session guardian thread waking up after %s seconds' % seconds, loglevel=log.loglevel_DEBUG)
 
