@@ -1,19 +1,19 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2011 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2012 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
-# Python X2go is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
+# Python X2Go is free software; you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 #
-# Python X2go is distributed in the hope that it will be useful,
+# Python X2Go is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program; if not, write to the
 # Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
@@ -36,19 +36,16 @@ import types
 import threading
 import gevent
 
-# Python X2go modules
+# Python X2Go modules
 import defaults
 import utils
 import log
 import mimeboxactions
 
-if defaults.X2GOCLIENT_OS != 'Windows':
-    from x2go_exceptions import WindowsError
-
 
 class X2goMIMEboxQueue(threading.Thread):
     """\
-    If the X2go MIME box is supported in a particaluar L{X2goSession} instance
+    If the X2Go MIME box is supported in a particaluar L{X2goSession} instance
     this class provides a sub-thread for handling incoming files in the MIME box
     directory. The actual handling of a dropped file is handled by the classes
     L{X2goMIMEboxActionOPEN}, L{X2goMIMEboxActionOPENWITH} and L{X2goMIMEboxActionSAVEAS}.
@@ -68,13 +65,13 @@ class X2goMIMEboxQueue(threading.Thread):
         @type profile_name: C{str}
         @param mimebox_dir: local directory for incoming MIME box files
         @type mimebox_dir: C{str}
-        @param mimebox_action: name or instance of either of the possible X2go print action classes
+        @param mimebox_action: name or instance of either of the possible X2Go print action classes
         @type mimebox_action: C{str} or instance
         @param client_instance: the underlying L{X2goClient} instance
-        @type client_instance: C{instance}
+        @type client_instance: C{obj}
         @param logger: you can pass an L{X2goLogger} object to the
             L{X2goPrintQueue} constructor
-        @type logger: C{instance}
+        @type logger: C{obj}
         @param loglevel: if no L{X2goLogger} object has been supplied a new one will be
             constructed with the given loglevel
         @type loglevel: C{int}
@@ -89,6 +86,7 @@ class X2goMIMEboxQueue(threading.Thread):
         self.profile_name = profile_name
         self.session_name = session_name
         self.mimebox_dir = mimebox_dir
+        if self.mimebox_dir: self.mimebox_dir = os.path.normpath(self.mimebox_dir)
         self.mimebox_extensions = mimebox_extensions
         self.client_instance = client_instance
         self.client_rootdir = client_instance.get_client_rootdir()
@@ -97,7 +95,7 @@ class X2goMIMEboxQueue(threading.Thread):
         self._accept_jobs = False
 
         if mimebox_action is None:
-            mimebox_action = mimebox_actions.X2goMIMEboxActionOPEN(client_instance=self.client_instance, logger=self.logger)
+            mimebox_action = mimeboxactions.X2goMIMEboxActionOPEN(client_instance=self.client_instance, logger=self.logger)
         elif type(mimebox_action) in (types.StringType, types.UnicodeType):
             mimebox_action = self.set_mimebox_action(mimebox_action, client_instance=self.client_instance, logger=self.logger)
         else:
@@ -128,7 +126,7 @@ class X2goMIMEboxQueue(threading.Thread):
 
     def resume(self):
         """\
-        Resume operation of the X2go MIME box queue and continue accepting new incoming 
+        Resume operation of the X2Go MIME box queue and continue accepting new incoming 
         files.
 
         """
@@ -164,6 +162,11 @@ class X2goMIMEboxQueue(threading.Thread):
         change of the MIME box action will be valid for the next incoming file in the MIME box
         directory.
 
+        @param mimebox_action: the MIME box action to execute for incoming files
+        @type mimebox_action: C{str} or C{obj}
+        @param kwargs: extra options for the specified MIME box action
+        @type kwargs: C{dict}
+
         """
         if mimebox_action in defaults.X2GO_MIMEBOX_ACTIONS.keys():
             mimebox_action = defaults.X2GO_MIMEBOX_ACTIONS[mimebox_action]
@@ -186,7 +189,7 @@ class X2goMIMEboxQueue(threading.Thread):
                 if self._incoming_mimebox_jobs:
 
                     for _job in self._incoming_mimebox_jobs:
-                        self.logger('processing incoming X2go MIME box job: %s' % _job, loglevel=log.loglevel_NOTICE)
+                        self.logger('processing incoming X2Go MIME box job: %s' % _job, loglevel=log.loglevel_NOTICE)
                         _new_mimeboxjob_thread = X2goMIMEboxJob(target=x2go_mimeboxjob_handler,
                                                                 kwargs={ 
                                                                   'mimebox_file': _job,
@@ -209,17 +212,17 @@ def x2go_mimeboxjob_handler(mimebox_file=None,
                             mimebox_action=None,
                             parent_thread=None, logger=None, ):
     """\
-    This function is called as a handler function for each incoming X2go MIME box file
+    This function is called as a handler function for each incoming X2Go MIME box file
     represented by the class L{X2goMIMEboxJob}.
 
-    @param mimebox_file: MIME box file name as placed in to the X2go MIME box spool directory
+    @param mimebox_file: MIME box file name as placed in to the X2Go MIME box spool directory
     @type mimebox_file: C{str}
     @param mimebox_action: an instance of either of the possible C{X2goMIMEboxActionXXX} classes
     @type mimebox_action: C{X2goMIMEboxActionXXX} nstance
     @param parent_thread: the L{X2goMIMEboxQueue} thread that actually created this handler's L{X2goMIMEboxJob} instance
-    @type parent_thread: C{instance}
+    @type parent_thread: C{obj}
     @param logger: the L{X2goMIMEboxQueue}'s logging instance
-    @type logger: C{instance}
+    @type logger: C{obj}
 
     """
     mimebox_action.profile_name = parent_thread.profile_name
@@ -237,7 +240,7 @@ def x2go_mimeboxjob_handler(mimebox_file=None,
     elif not _blacklisted and not _dotfile:
         logger('file extension of MIME box file %s is prohibited by session profile configuration' % mimebox_file, loglevel=log.loglevel_NOTICE)
     elif _dotfile:
-        logger('placing files starting with a dot (.<file>) into the X2go MIME box is prohibited, ignoring the file ,,%s\'\'' % mimebox_file, loglevel=log.loglevel_WARN)
+        logger('placing files starting with a dot (.<file>) into the X2Go MIME box is prohibited, ignoring the file ,,%s\'\'' % mimebox_file, loglevel=log.loglevel_WARN)
     else:
         logger('file extension of MIME box file %s has been found in Python X2go\' hardcoded MIME box extenstions blacklist' % mimebox_file, loglevel=log.loglevel_WARN)
 
@@ -256,7 +259,7 @@ def x2go_mimeboxjob_handler(mimebox_file=None,
 
 class X2goMIMEboxJob(threading.Thread):
     """\
-    For each X2go MIME box job we create a sub-thread that let's 
+    For each X2Go MIME box job we create a sub-thread that let's 
     the MIME box job be processed in the background.
 
     As a handler for this class the function L{x2go_mimeboxjob_handler()} 
@@ -265,7 +268,7 @@ class X2goMIMEboxJob(threading.Thread):
     """
     def __init__(self, **kwargs):
         """\
-        Construct the X2go MIME box job thread...
+        Construct the X2Go MIME box job thread...
 
         All parameters (**kwargs) are passed through to the constructor
         of C{threading.Thread()}.
