@@ -1,19 +1,19 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2011 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2012 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
-# Python X2go is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
+# Python X2Go is free software; you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 #
-# Python X2go is distributed in the hope that it will be useful,
+# Python X2Go is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program; if not, write to the
 # Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
@@ -30,14 +30,10 @@ __NAME__ = 'x2goprintactions-pylib'
 
 # modules
 import os
-import sys
 import shutil
 import copy
-import types
-import threading
 import time
-import re
-import string
+import gevent
 
 from defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
 if _X2GOCLIENT_OS in ("Windows"):
@@ -46,8 +42,10 @@ if _X2GOCLIENT_OS in ("Windows"):
     import win32print
 else:
     import gevent_subprocess as subprocess
+    import x2go_exceptions
+    WindowsError = x2go_exceptions.WindowsError
 
-# Python X2go modules
+# Python X2Go modules
 import log
 import defaults
 # we hide the default values from epydoc (that's why we transform them to _UNDERSCORE variables)
@@ -65,13 +63,13 @@ class X2goPrintAction(object):
     def __init__(self, client_instance=None, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
         This is a meta class and has no functionality as such. It is used as parent 
-        class by »real« X2go print actions.
+        class by »real« X2Go print actions.
 
         @param client_instance: the underlying L{X2goClient} instance
-        @type client_instance: C{instance}
+        @type client_instance: C{obj}
         @param logger: you can pass an L{X2goLogger} object to the
             L{X2goPrintAction} constructor
-        @type logger: C{instance}
+        @type logger: C{obj}
         @param loglevel: if no L{X2goLogger} object has been supplied a new one will be
             constructed with the given loglevel
         @type loglevel: C{int}
@@ -90,40 +88,58 @@ class X2goPrintAction(object):
         self.client_instance = client_instance
 
     @property
-    def name():
+    def name(self):
         """\
-        Return the X2go print action's name.
+        Return the X2Go print action's name.
 
         """
         return self.__name__
 
     @property
-    def description():
+    def description(self):
         """\
-        Return the X2go print action's description text.
+        Return the X2Go print action's description text.
 
         """
         return self.__description__
 
-    def do_print(self, pdf_file, job_title, spool_dir, ):
-        """\
+    def _do_print(self, pdf_file, job_title, spool_dir, ):
+        """
         Perform the defined print action (doing nothing in L{X2goPrintAction} parent class).
 
-        @param pdf_file: PDF file name as placed in to the X2go spool directory
+        @param pdf_file: PDF file name as placed in to the X2Go spool directory
         @type pdf_file: C{str}
         @param job_title: human readable print job title
         @type job_title: C{str}
-        @param spool_dir: location of the X2go client's spool directory
+        @param spool_dir: location of the X2Go client's spool directory
         @type spool_dir: C{str}
 
         """
         pass
 
+    def do_print(self, pdf_file, job_title, spool_dir, ):
+        """\
+        Wrap around the actual print action (C{self._do_print}) with
+        gevent.spawn().
+
+        @param pdf_file: PDF file name as placed in to the X2Go spool directory
+        @type pdf_file: C{str}
+        @param job_title: human readable print job title
+        @type job_title: C{str}
+        @param spool_dir: location of the X2Go client's spool directory
+        @type spool_dir: C{str}
+
+        """
+        pdf_file = os.path.normpath(pdf_file)
+        spool_dir = os.path.normpath(spool_dir)
+
+        self._do_print(pdf_file, job_title, spool_dir)
+
     def _humanreadable_filename(self, pdf_file, job_title, target_path):
         """\
-        Extract a human readable filename for the X2go print job file.
+        Extract a human readable filename for the X2Go print job file.
 
-        @param pdf_file: PDF file name as placed in to the X2go spool directory
+        @param pdf_file: PDF file name as placed in to the X2Go spool directory
         @type pdf_file: C{str}
         @param job_title: human readable print job title
         @type job_title: C{str}
@@ -133,12 +149,12 @@ class X2goPrintAction(object):
         @rtype: C{str}
 
         """
-        _hr_path = os.path.expanduser(os.path.join(target_path, '%s.pdf' % utils.slugify(job_title)))
+        _hr_path = os.path.normpath(os.path.expanduser(os.path.join(os.path.normpath(target_path), '%s.pdf' % utils.slugify(job_title))))
         i = 0
 
         while os.path.exists(_hr_path):
             i += 1
-            _hr_path = os.path.expanduser(os.path.join(target_path, '%s(%s).pdf' % (utils.slugify(job_title), i)))
+            _hr_path = os.path.normpath(os.path.expanduser(os.path.join(os.path.normpath(target_path), '%s(%s).pdf' % (utils.slugify(job_title), i))))
 
         return _hr_path
 
@@ -156,12 +172,12 @@ class X2goPrintActionPDFVIEW(X2goPrintAction):
     def __init__(self, client_instance=None, pdfview_cmd=None, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
         @param client_instance: the underlying L{X2goClient} instance
-        @type client_instance: C{instance}
+        @type client_instance: C{obj}
         @param pdfview_cmd: command that starts the external PDF viewer application
         @type pdfview_cmd: C{str}
         @param logger: you can pass an L{X2goLogger} object to the
             L{X2goPrintActionPDFVIEW} constructor
-        @type logger: C{instance}
+        @type logger: C{obj}
         @param loglevel: if no L{X2goLogger} object has been supplied a new one will be
             constructed with the given loglevel
         @type loglevel: C{int}
@@ -172,22 +188,27 @@ class X2goPrintActionPDFVIEW(X2goPrintAction):
         self.pdfview_cmd = pdfview_cmd
         X2goPrintAction.__init__(self, client_instance=client_instance, logger=logger, loglevel=loglevel)
 
-    def do_print(self, pdf_file, job_title, spool_dir, ):
+    def _do_print(self, pdf_file, job_title, spool_dir, ):
         """\
-        Open an incoming X2go print job (PDF file) in an external PDF viewer application.
+        Open an incoming X2Go print job (PDF file) in an external PDF viewer application.
 
-        @param pdf_file: PDF file name as placed in to the X2go spool directory
+        @param pdf_file: PDF file name as placed in to the X2Go spool directory
         @type pdf_file: C{str}
         @param job_title: human readable print job title
         @type job_title: C{str}
-        @param spool_dir: location of the X2go client's spool directory
+        @param spool_dir: location of the X2Go client's spool directory
         @type spool_dir: C{str}
 
+        @raise OSError: pass through all C{OSError}s except no. 2
+
         """
+        pdf_file = os.path.normpath(pdf_file)
+        spool_dir = os.path.normpath(spool_dir)
+
         if _X2GOCLIENT_OS == "Windows":
             self.logger('viewing incoming job in PDF viewer with Python\'s os.startfile( command): %s' % pdf_file, loglevel=log.loglevel_DEBUG)
             try:
-                os.startfile(pdf_file)
+                gevent.spawn(os.startfile, pdf_file)
             except WindowsError, win_err:
                 if self.client_instance:
                     self.client_instance.HOOK_printaction_error(pdf_file,
@@ -204,11 +225,11 @@ class X2goPrintActionPDFVIEW(X2goPrintAction):
             cmd_line = [ self.pdfview_cmd, _hr_filename, ]
             self.logger('viewing incoming PDF with command: %s' % ' '.join(cmd_line), loglevel=log.loglevel_DEBUG)
             try:
-                p = subprocess.Popen(cmd_line, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=_PRINT_ENV)
+                subprocess.Popen(cmd_line, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=_PRINT_ENV)
             except OSError, e:
                 if e.errno == 2:
                     cmd_line = [ defaults.DEFAULT_PDFVIEW_CMD, _hr_filename ]
-                    p = subprocess.Popen(cmd_line, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=_PRINT_ENV)
+                    subprocess.Popen(cmd_line, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=_PRINT_ENV)
                 else:
                     raise(e)
             self.logger('waiting 20s longer before deleting the PDF file ,,%s\'\'' % _hr_filename, loglevel=log.loglevel_DEBUG)
@@ -229,12 +250,12 @@ class X2goPrintActionPDFSAVE(X2goPrintAction):
     def __init__(self, client_instance=None, save_to_folder=None, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
         @param client_instance: the underlying L{X2goClient} instance
-        @type client_instance: C{instance}
+        @type client_instance: C{obj}
         @param save_to_folder: saving location for incoming print jobs (PDF files)
         @type save_to_folder: C{str}
         @param logger: you can pass an L{X2goLogger} object to the
             L{X2goPrintActionPDFSAVE} constructor
-        @type logger: C{instance}
+        @type logger: C{obj}
         @param loglevel: if no L{X2goLogger} object has been supplied a new one will be
             constructed with the given loglevel
         @type loglevel: C{int}
@@ -254,18 +275,21 @@ class X2goPrintActionPDFSAVE(X2goPrintAction):
         if not os.path.exists(self.save_to_folder):
             os.makedirs(self.save_to_folder, mode=0755)
 
-    def do_print(self, pdf_file, job_title, spool_dir):
+    def _do_print(self, pdf_file, job_title, spool_dir):
         """\
-        Save an incoming X2go print job (PDF file) to a local folder.
+        Save an incoming X2Go print job (PDF file) to a local folder.
 
-        @param pdf_file: PDF file name as placed in to the X2go spool directory
+        @param pdf_file: PDF file name as placed in to the X2Go spool directory
         @type pdf_file: C{str}
         @param job_title: human readable print job title
         @type job_title: C{str}
-        @param spool_dir: location of the X2go client's spool directory
+        @param spool_dir: location of the X2Go client's spool directory
         @type spool_dir: C{str}
 
         """
+        pdf_file = os.path.normpath(pdf_file)
+        spool_dir = os.path.normpath(spool_dir)
+
         dest_file = self._humanreadable_filename(pdf_file, job_title, target_path=self.save_to_folder)
         shutil.copy2(pdf_file, dest_file)
 
@@ -281,12 +305,12 @@ class X2goPrintActionPRINT(X2goPrintAction):
     def __init__(self, client_instance=None, printer=None, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
         @param client_instance: the underlying L{X2goClient} instance
-        @type client_instance: C{instance}
+        @type client_instance: C{obj}
         @param printer: name of the preferred printer, if C{None} the system's/user's default printer will be used
         @type printer: C{str}
         @param logger: you can pass an L{X2goLogger} object to the
             L{X2goPrintActionPRINT} constructor
-        @type logger: C{instance}
+        @type logger: C{obj}
         @param loglevel: if no L{X2goLogger} object has been supplied a new one will be
             constructed with the given loglevel
         @type loglevel: C{int}
@@ -295,18 +319,21 @@ class X2goPrintActionPRINT(X2goPrintAction):
         self.printer = printer
         X2goPrintAction.__init__(self, client_instance=client_instance, logger=logger, loglevel=loglevel)
 
-    def do_print(self, pdf_file, job_title, spool_dir, ):
+    def _do_print(self, pdf_file, job_title, spool_dir, ):
         """\
-        Actually really print an incoming X2go print job (PDF file) to a local printer device.
+        Actually really print an incoming X2Go print job (PDF file) to a local printer device.
 
-        @param pdf_file: PDF file name as placed in to the X2go spool directory
+        @param pdf_file: PDF file name as placed in to the X2Go spool directory
         @type pdf_file: C{str}
         @param job_title: human readable print job title
         @type job_title: C{str}
-        @param spool_dir: location of the X2go client's spool directory
+        @param spool_dir: location of the X2Go client's spool directory
         @type spool_dir: C{str}
 
         """
+        pdf_file = os.path.normpath(pdf_file)
+        spool_dir = os.path.normpath(spool_dir)
+
         _hr_filename = self._humanreadable_filename(pdf_file, job_title, spool_dir)
         if _X2GOCLIENT_OS == 'Windows':
             _default_printer = win32print.GetDefaultPrinter()
@@ -328,12 +355,12 @@ class X2goPrintActionPRINT(X2goPrintAction):
                     _gsprint_bin = os.path.normpath(os.path.join(_program_files, 'ghostgum', 'gsview', 'gsprint.exe',))
                     self.logger('Using hard-coded gsprint.exe path: %s' % _gsprint_bin, loglevel=log.loglevel_DEBUG)
                 self.logger('Trying Ghostgum tool ,,gsprint.exe'' for printing first (full path: %s)' % _gsprint_bin, loglevel=log.loglevel_DEBUG)
-                p = subprocess.Popen([_gsprint_bin, pdf_file, ],
-                                     stdin=_stdin,
-                                     stdout=subprocess.PIPE,
-                                     stderr=subprocess.STDOUT,
-                                     shell=_shell,
-                                    )
+                subprocess.Popen([_gsprint_bin, pdf_file, ],
+                                  stdin=_stdin,
+                                  stdout=subprocess.PIPE,
+                                  stderr=subprocess.STDOUT,
+                                  shell=_shell,
+                                 )
                 # give gsprint.exe a little time to find our printer
                 time.sleep(10)
 
@@ -384,7 +411,7 @@ class X2goPrintActionPRINT(X2goPrintAction):
                              '%s' % _hr_filename,
                            ]
             self.logger('executing local print command: %s' % " ".join(cmd_line), loglevel=log.loglevel_DEBUG)
-            p = subprocess.Popen(cmd_line, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=_PRINT_ENV)
+            subprocess.Popen(cmd_line, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=_PRINT_ENV)
 
             # this is nasty!!!!
             self.logger('waiting 20s longer before deleting the PDF file ,,%s\'\'' % _hr_filename, loglevel=log.loglevel_DEBUG)
@@ -407,12 +434,12 @@ class X2goPrintActionPRINTCMD(X2goPrintAction):
     def __init__(self, client_instance=None, print_cmd=None, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
         @param client_instance: the underlying L{X2goClient} instance
-        @type client_instance: C{instance}
+        @type client_instance: C{obj}
         @param print_cmd: external command to be called on incoming print jobs
         @type print_cmd: C{str}
         @param logger: you can pass an L{X2goLogger} object to the
             L{X2goPrintActionPRINTCMD} constructor
-        @type logger: C{instance}
+        @type logger: C{obj}
         @param loglevel: if no L{X2goLogger} object has been supplied a new one will be
             constructed with the given loglevel
         @type loglevel: C{int}
@@ -423,26 +450,29 @@ class X2goPrintActionPRINTCMD(X2goPrintAction):
         self.print_cmd = print_cmd
         X2goPrintAction.__init__(self, client_instance=client_instance, logger=logger, loglevel=loglevel)
 
-    def do_print(self, pdf_file, job_title, spool_dir):
+    def _do_print(self, pdf_file, job_title, spool_dir):
         """\
         Execute an external command that has been defined on construction 
         of this L{X2goPrintActionPRINTCMD} instance.
 
-        @param pdf_file: PDF file name as placed in to the X2go spool directory
+        @param pdf_file: PDF file name as placed in to the X2Go spool directory
         @type pdf_file: C{str}
         @param job_title: human readable print job title
         @type job_title: C{str}
-        @param spool_dir: location of the X2go client's spool directory
+        @param spool_dir: location of the X2Go client's spool directory
         @type spool_dir: C{str}
 
         """
+        pdf_file = os.path.normpath(pdf_file)
+        spool_dir = os.path.normpath(spool_dir)
+
         _hr_filename = self._humanreadable_filename(pdf_file, job_title, spool_dir)
         shutil.copy2(pdf_file, _hr_filename)
         self.logger('executing external command ,,%s\'\' on PDF file %s' % (self.print_cmd, _hr_filename), loglevel=log.loglevel_NOTICE)
         cmd_line = self.print_cmd.split()
         cmd_line.append(_hr_filename)
         self.logger('executing external command: %s' % " ".join(cmd_line), loglevel=log.loglevel_DEBUG)
-        p = subprocess.Popen(cmd_line, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=_PRINT_ENV)
+        subprocess.Popen(cmd_line, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=_PRINT_ENV)
 
         # this is nasty!!!!
         self.logger('waiting 20s longer before deleting the PDF file ,,%s\'\'' % _hr_filename, loglevel=log.loglevel_DEBUG)
@@ -466,34 +496,39 @@ class X2goPrintActionDIALOG(X2goPrintAction):
         @param client_instance: an L{X2goClient} instance, within your customized L{X2goClient} make sure 
             you have a C{HOOK_open_print_dialog(filename=<str>)} method defined that will actually
             open the print dialog.
-        @type client_instance: C{instance}
+        @type client_instance: C{obj}
         @param logger: you can pass an L{X2goLogger} object to the
             L{X2goPrintActionDIALOG} constructor
-        @type logger: C{instance}
+        @type logger: C{obj}
         @param loglevel: if no L{X2goLogger} object has been supplied a new one will be
             constructed with the given loglevel
         @type loglevel: C{int}
 
+        @raise X2goPrintActionException: if the client_instance has not been passed to the DIALOG print action
+
         """
         if client_instance is None:
             raise x2go_exceptions.X2goPrintActionException('the DIALOG print action needs to know the X2goClient instance (client=<instance>)')
         X2goPrintAction.__init__(self, client_instance=client_instance, logger=logger, loglevel=loglevel)
 
-    def do_print(self, pdf_file, job_title, spool_dir):
+    def _do_print(self, pdf_file, job_title, spool_dir):
         """\
         Execute an external command that has been defined on construction 
         of this L{X2goPrintActionPRINTCMD} instance.
 
-        @param pdf_file: PDF file name as placed in to the X2go spool directory
+        @param pdf_file: PDF file name as placed in to the X2Go spool directory
         @type pdf_file: C{str}
         @param job_title: human readable print job title
         @type job_title: C{str}
-        @param spool_dir: location of the X2go client's spool directory
+        @param spool_dir: location of the X2Go client's spool directory
         @type spool_dir: C{str}
 
         """
+        pdf_file = os.path.normpath(pdf_file)
+        spool_dir = os.path.normpath(spool_dir)
+
         self.logger('Session %s (%s) is calling X2goClient class hook method <client_instance>.HOOK_open_print_dialog' % (self.session_name, self.profile_name), loglevel=log.loglevel_NOTICE)
         _new_print_action = self.client_instance.HOOK_open_print_dialog(profile_name=self.profile_name, session_name=self.session_name)
         if type(_new_print_action) != type(self):
-            _new_print_action.do_print(pdf_file, job_title, spool_dir)
+            _new_print_action._do_print(pdf_file, job_title, spool_dir)
 
