@@ -27,9 +27,12 @@ import os
 import copy
 import types
 import time
+import threading
+import re
 
 # Python X2Go modules
 import log
+import utils
 import session
 import x2go_exceptions
 
@@ -45,6 +48,7 @@ from x2go.backends.printing import X2goClientPrinting as _X2goClientPrinting
 from defaults import LOCAL_HOME as _LOCAL_HOME
 from defaults import X2GO_CLIENT_ROOTDIR as _X2GO_CLIENT_ROOTDIR
 from defaults import X2GO_SESSIONS_ROOTDIR as _X2GO_SESSIONS_ROOTDIR
+from defaults import X2GO_SESSIONPROFILE_DEFAULTS as _X2GO_SESSIONPROFILE_DEFAULTS
 from defaults import X2GO_SSH_ROOTDIR as _X2GO_SSH_ROOTDIR
 
 class X2goSessionRegistry(object):
@@ -79,6 +83,7 @@ class X2goSessionRegistry(object):
 
         self._last_available_session_registration = None
         self._skip_auto_registration = False
+        self._profile_locks = {}
 
     def keys(self):
         """\
@@ -94,7 +99,8 @@ class X2goSessionRegistry(object):
         result = 'X2goSessionRegistry('
         for p in dir(self):
             if '__' in p or not p in self.__dict__ or type(p) is types.InstanceType: continue
-            result += p + '=' + str(self.__dict__[p])
+            result += p + '=' + str(self.__dict__[p]) + ','
+        result = result.strip(',')
         return result + ')'
 
     def __call__(self, session_uuid):
@@ -267,7 +273,8 @@ class X2goSessionRegistry(object):
 
             if not self(_session_uuid).update_status(session_list=session_list, force_update=force_update):
                 # skip this run, as nothing has changed since the last time...
-                return False
+                continue
+
             _last_status = copy.deepcopy(self(_session_uuid)._last_status)
             _current_status = copy.deepcopy(self(_session_uuid)._current_status)
 
@@ -316,8 +323,8 @@ class X2goSessionRegistry(object):
                         if _last_status['suspended']:
                             # from a suspended state
                             self.client_instance.HOOK_on_session_has_resumed_by_other(session_uuid=_session_uuid, profile_name=_profile_name, session_name=_session_name)
-                        elif _last_status['virgin']:
-                            # as a new session
+                        elif _last_status['connected'] and _last_status['virgin']:
+                            # as a new session, do not report directly after connect due to many false positives then...
                             self.client_instance.HOOK_on_session_has_started_by_other(session_uuid=_session_uuid, profile_name=_profile_name, session_name=_session_name)
 
             elif _last_status['connected'] and (not _last_status['suspended'] and _current_status['suspended']) and not _current_status['faulty'] and _session_name:
@@ -325,6 +332,11 @@ class X2goSessionRegistry(object):
                 # unregister as master session
                 if _profile_name in self.master_sessions.keys():
                     if self.master_sessions[_profile_name] == self(_session_uuid):
+
+                        # save exported folders to session profile config if requested by session profile parameter ,,restoreexports''...
+                        if self.client_instance and self(_session_uuid).restore_shared_local_folders:
+                            self.client_instance.set_profile_config(_profile_name, 'export', self(_session_uuid)._restore_exported_folders)
+
                         self(_session_uuid).unset_master_session()
                         del self.master_sessions[_profile_name]
 
@@ -337,6 +349,11 @@ class X2goSessionRegistry(object):
                 # unregister as master session
                 if _profile_name in self.master_sessions.keys():
                     if self.master_sessions[_profile_name] == self(_session_uuid):
+
+                        # save exported folders to session profile config if requested by session profile parameter ,,restoreexports''...
+                        if self.client_instance and self(_session_uuid).restore_shared_local_folders:
+                            self.client_instance.set_profile_config(_profile_name, 'export', self(_session_uuid)._restore_exported_folders)
+
                         self(_session_uuid).unset_master_session()
                         del self.master_sessions[_profile_name]
 
@@ -369,7 +386,7 @@ class X2goSessionRegistry(object):
 
         return True
 
-    def register_available_server_sessions(self, profile_name, session_list=None, newly_connected=False, re_register=False):
+    def register_available_server_sessions(self, profile_name, session_list=None, newly_connected=False, re_register=False, skip_pubapp_sessions=False):
         """\
         Register server-side available X2Go sessions with this L{X2goSessionRegistry} instance for a given profile name.
 
@@ -382,6 +399,8 @@ class X2goSessionRegistry(object):
         @type newly_connected: C{bool}
         @param re_register: re-register available sessions, needs to be done after changes to the session profile
         @type re_register: C{bool}
+        @param skip_pubapp_sessions: Do not register published applications sessions
+        @type skip_pubapp_sessions: C{bool}
 
         """
         if self._last_available_session_registration is not None:
@@ -438,12 +457,13 @@ class X2goSessionRegistry(object):
 
                     # this if clause catches problems when x2golistsessions commands give weird results
                     if not self.has_session_of_session_name(session_name) or re_register:
-                        session_uuid = self.register(server, profile_id, profile_name,
-                                                     session_name=session_name, virgin=False,
-                                                     **kwargs
-                                                    )
-                        self(session_uuid).connected = True
-                        self.update_status(session_uuid=session_uuid, force_update=True, newly_connected=newly_connected)
+                        if not (skip_pubapp_sessions and re.match('.*_stRPUBLISHED_.*', session_name)):
+                            session_uuid = self.register(server, profile_id, profile_name,
+                                                         session_name=session_name, virgin=False,
+                                                         **kwargs
+                                                        )
+                            self(session_uuid).connected = True
+                            self.update_status(session_uuid=session_uuid, force_update=True, newly_connected=newly_connected)
 
     def register(self, server, profile_id, profile_name,
                  session_name=None,
@@ -505,40 +525,52 @@ class X2goSessionRegistry(object):
         @rtype: C{str}
 
         """
+        if profile_id not in self._profile_locks.keys():
+            self._profile_locks[profile_id] = threading.Lock()
+
+        self._profile_locks[profile_id].acquire()
+
         control_session = None
         if profile_id in self.control_sessions.keys():
             control_session = self.control_sessions[profile_id]
 
+        try:
+            _params = self.client_instance.session_profiles.to_session_params(profile_id)
+
+        except x2go_exceptions.X2goProfileException:
+            _params = utils._convert_SessionProfileOptions_2_SessionParams(_X2GO_SESSIONPROFILE_DEFAULTS)
+
+        for _k in _params.keys():
+            if _k in kwargs.keys():
+                _params[_k] = kwargs[_k]
+
         # when starting a new session, we will try to use unused registered virgin sessions
         # depending on your application layout, there should either be one or no such virgin session at all
         _virgin_sessions = [ s for s in self.virgin_sessions_of_profile_name(profile_name, return_objects=True) if not s.activated ]
         if _virgin_sessions and not session_name:
-
-            _virgin_sessions[0].activated = True
             session_uuid = _virgin_sessions[0].get_uuid()
-            _params = self.client_instance.session_profiles.to_session_params(profile_id)
-            for _k in _params.keys():
-                if _k in kwargs.keys():
-                    _params[_k] = kwargs[_k]
-            self(session_uuid).update_params(_params)
-            self(session_uuid).set_server(server)
-            self(session_uuid).set_profile_name(profile_name)
+            self(session_uuid).activated = True
             self.logger('using already initially-registered yet-unused session %s' % session_uuid, loglevel=log.loglevel_NOTICE)
-            return session_uuid
 
-        session_uuid = self.get_session_of_session_name(session_name)
+        else:
+            session_uuid = self.get_session_of_session_name(session_name)
+            if session_uuid is not None: self.logger('using already registered-by-session-name session %s' % session_uuid, loglevel=log.loglevel_NOTICE)
+
         if session_uuid is not None:
             self(session_uuid).activated = True
-            _params = self.client_instance.session_profiles.to_session_params(profile_id)
-            for _k in _params.keys():
-                if _k in kwargs.keys():
-                    _params[_k] = kwargs[_k]
             self(session_uuid).update_params(_params)
             self(session_uuid).set_server(server)
             self(session_uuid).set_profile_name(profile_name)
-            self.logger('using already registered-by-session-name session %s' % session_uuid, loglevel=log.loglevel_NOTICE)
+            self._profile_locks[profile_id].release()
             return session_uuid
 
+        try: del _params['server'] 
+        except: pass
+        try: del _params['profile_name']
+        except: pass
+        try: del _params['profile_id'] 
+        except: pass
+
         s = session.X2goSession(server=server, control_session=control_session,
                                 profile_id=profile_id, profile_name=profile_name,
                                 session_name=session_name,
@@ -555,7 +587,8 @@ class X2goSessionRegistry(object):
                                 keep_controlsession_alive=keep_controlsession_alive,
                                 add_to_known_hosts=add_to_known_hosts,
                                 known_hosts=known_hosts,
-                                logger=self.logger, **kwargs)
+                                client_instance=self.client_instance,
+                                logger=self.logger, **_params)
 
         session_uuid = s._X2goSession__get_uuid()
         self.logger('registering X2Go session %s...' % profile_name, log.loglevel_NOTICE)
@@ -570,6 +603,7 @@ class X2goSessionRegistry(object):
         if control_session is None:
             self(session_uuid).do_auto_connect()
 
+        self._profile_locks[profile_id].release()
         return session_uuid
 
     def has_session_of_session_name(self, session_name):
