@@ -1,25 +1,25 @@
 #!/usr/bin/env python
 
-# Copyright (C) 2010-2011 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2012 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
-# Python X2go is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
+# Python X2Go is free software; you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 #
-# Python X2go is distributed in the hope that it will be useful,
+# Python X2Go is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program; if not, write to the
 # Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
 """\
-X2go reverse SSH/Paramiko tunneling provides X2go sound, X2go printing and
-X2go sshfs for folder sharing and mounting remote devices in X2go terminal
+X2Go reverse SSH/Paramiko tunneling provides X2Go sound, X2Go printing and
+X2Go sshfs for folder sharing and mounting remote devices in X2Go terminal
 server sessions.
 
 """
@@ -33,15 +33,14 @@ import paramiko
 
 # gevent/greenlet
 from gevent import select, socket, Timeout
-from gevent import sleep as gevent_sleep
 
-# Python X2go modules
+# Python X2Go modules
 import log
 
 
 def x2go_transport_tcp_handler(chan, (origin_addr, origin_port), (server_addr, server_port)):
     """\
-    An X2go customized TCP handler for the Paramiko/SSH C{Transport()} class.
+    An X2Go customized TCP handler for the Paramiko/SSH C{Transport()} class.
 
     Incoming channels will be put into Paramiko's default accept queue. This corresponds to 
     the default behaviour of Paramiko's C{Transport} class.
@@ -74,7 +73,7 @@ def x2go_transport_tcp_handler(chan, (origin_addr, origin_port), (server_addr, s
 class X2goRevFwTunnel(threading.Thread):
     """\
     L{X2goRevFwTunnel} class objects are used to reversely tunnel 
-    X2go audio, X2go printing and X2go folder sharing / device mounting
+    X2Go audio, X2Go printing and X2Go folder sharing / device mounting
     through Paramiko/SSH.
 
     """
@@ -87,13 +86,13 @@ class X2goRevFwTunnel(threading.Thread):
         notification (L{X2goRevFwTunnel.notify()} gets called from within the transport's 
         TCP handler function L{x2go_transport_tcp_handler} of the L{X2goSession} instance.
 
-        @param server_port: the TCP/IP port on the X2go server (starting point of the tunnel), 
+        @param server_port: the TCP/IP port on the X2Go server (starting point of the tunnel), 
             normally some number above 30000
         @type server_port: int
-        @param remote_host: the target address for reversely tunneled traffic. With X2go this should 
+        @param remote_host: the target address for reversely tunneled traffic. With X2Go this should 
             always be set to the localhost (IPv4) address.
         @type remote_host: str
-        @param remote_port: the TCP/IP port on the X2go client (end point of the tunnel),
+        @param remote_port: the TCP/IP port on the X2Go client (end point of the tunnel),
             normally an application's standard port (22 for SSH, 4713 for pulse audio, etc.)
         @type remote_port: int
         @param ssh_transport: the L{X2goSession}'s Paramiko/SSH transport instance
@@ -147,7 +146,7 @@ class X2goRevFwTunnel(threading.Thread):
         timeout = Timeout(10)
         timeout.start()
         try:
-            self.ssh_transport.cancel_port_forward('', self.server_port)
+            self.ssh_transport.global_request('cancel-tcpip-forward', (address, port), wait=True)
         except:
             pass
         finally:
@@ -228,14 +227,14 @@ class X2goRevFwTunnel(threading.Thread):
 
         L{X2goRevFwTunnel.run()} waits for notifications of an appropriate incoming
         Paramiko/SSH channel (issued by L{X2goRevFwTunnel.notify()}). Appropriate in
-        this context means, that its start point on the X2go server matches the class's
+        this context means, that its start point on the X2Go server matches the class's
         property C{server_port}.
 
         Once a new incoming channel gets announced by the L{notify()} method, a new 
         L{X2goRevFwChannelThread} instance will be initialized. As a data stream handler,
         the function L{x2go_rev_forward_channel_handler()} will be used.
 
-        The channel will last till the connection gets dropped on the X2go server side or 
+        The channel will last till the connection gets dropped on the X2Go server side or 
         until the tunnel gets paused by an L{X2goRevFwTunnel.pause()} call or stopped via the
         L{X2goRevFwTunnel.stop_thread()} method.
 
@@ -246,11 +245,11 @@ class X2goRevFwTunnel(threading.Thread):
 
             self.incoming_channel.acquire()
 
-            self.logger('waiting for incoming data channel on X2go server port: [127.0.0.1]:%s' % self.server_port, loglevel=log.loglevel_DEBUG)
+            self.logger('waiting for incoming data channel on X2Go server port: [127.0.0.1]:%s' % self.server_port, loglevel=log.loglevel_DEBUG)
             self.incoming_channel.wait()
 
             if self._keepalive:
-                self.logger('detected incoming data channel on X2go server port: [127.0.0.1]:%s' % self.server_port, loglevel=log.loglevel_DEBUG)
+                self.logger('detected incoming data channel on X2Go server port: [127.0.0.1]:%s' % self.server_port, loglevel=log.loglevel_DEBUG)
                 _chan = self.ssh_transport.accept()
                 self.logger('data channel %s for server port [127.0.0.1]:%s is up' % (_chan, self.server_port), loglevel=log.loglevel_DEBUG)
             else:
@@ -284,7 +283,7 @@ def x2go_rev_forward_channel_handler(chan=None, addr='', port=0, parent_thread=N
         - ... if the L{X2goRevFwTunnel} parent thread gets paused. The call
         of L{X2goRevFwTunnel.pause()} on the instance can be used to shut down all incoming 
         tunneled SSH connections associated to this L{X2goRevFwTunnel} instance
-        from within a Python X2go application.
+        from within a Python X2Go application.
 
     @param chan: channel
     @type chan: C{class}
