@@ -33,6 +33,7 @@ import socket
 import gevent
 import string
 import subprocess
+import distutils.version
 
 # Python X2Go modules
 from defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
@@ -185,6 +186,7 @@ def _convert_SessionProfileOptions_2_SessionParams(options):
             'speed': 'link',
             'sshport': 'port',
             'useexports': 'allow_share_local_folders',
+            'restoreexports': 'restore_shared_local_folders',
             'usemimebox': 'allow_mimebox',
             'mimeboxextensions': 'mimebox_extensions',
             'mimeboxaction': 'mimebox_action',
@@ -200,6 +202,7 @@ def _convert_SessionProfileOptions_2_SessionParams(options):
             'iconvfrom': 'client_encoding',
             'usesshproxy': 'use_sshproxy',
             'sshproxyhost': 'sshproxy_host',
+            'sshproxyport': 'sshproxy_port',
             'sshproxyuser': 'sshproxy_user',
             'sshproxykeyfile': 'sshproxy_key_filename',
             'sshproxytunnel': 'sshproxy_tunnel',
@@ -207,8 +210,10 @@ def _convert_SessionProfileOptions_2_SessionParams(options):
             'setsessiontitle': 'set_session_title',
             'published': 'published_applications',
             'autostart': 'auto_start_or_resume',
-            'autologin': 'auto_connect',
-
+            'autoconnect': 'auto_connect',
+            'forwardsshagent': 'forward_sshagent',
+            'autologin': 'look_for_keys',
+            'sshproxyautologin': 'sshproxy_look_for_keys',
     }
     _speed_dict = {
             '0': 'modem',
@@ -246,6 +251,12 @@ def _convert_SessionProfileOptions_2_SessionParams(options):
                 else:
                     _params[opt] = []
 
+    if _params['cmd'] == "XFCE4": _params['cmd'] = "XFCE"
+    if _params['look_for_keys']:
+        _params['allow_agent'] = True
+    if _params['sshproxy_look_for_keys']:
+        _params['sshproxy_allow_agent'] = True
+
     # append value for quality to value for pack method
     if _params['quality']:
         _params['pack'] = '%s-%s' % (_params['pack'], _params['quality'])
@@ -271,13 +282,16 @@ def _convert_SessionProfileOptions_2_SessionParams(options):
             if _shared_folder.split(":")[-1] == "1":
                 _params['share_local_folders'].append(":".join(_shared_folder.split(":")[:-1]))
 
-    if not options['fullscreen']:
-        _params['geometry'] = '%sx%s' % (options['width'], options['height'])
-    else:
+    if options['fullscreen']:
         _params['geometry'] = 'fullscreen'
+    elif options['maxdim']:
+        _params['geometry'] = 'maximize'
+    else:
+        _params['geometry'] = '%sx%s' % (options['width'], options['height'])
     del _params['width']
     del _params['height']
     del _params['fullscreen']
+    del _params['maxdim']
 
     if not options['sound']:
         _params['snd_system'] = 'none'
@@ -306,6 +320,14 @@ def _convert_SessionProfileOptions_2_SessionParams(options):
         del _params['dpi']
     del _params['setdpi']
 
+    if options['sshproxysameuser']:
+        _params['sshproxy_user'] = _params['username']
+    del _params['sshproxysameuser']
+    if options['sshproxysamepass']:
+        _params['sshproxy_reuse_authinfo'] = True
+        _params['sshproxy_key_filename'] = _params['key_filename']
+    del _params['sshproxysamepass']
+
     # currently known but ignored in Python X2go
     _ignored_options = [
             'startsoundsystem',
@@ -315,6 +337,11 @@ def _convert_SessionProfileOptions_2_SessionParams(options):
             'xinerama',
             'multidisp',
             'krblogin',
+            'directrdp',
+            'directrdpsettings',
+            'rdpclient',
+            'rdpport',
+            'sshproxytype',
     ]
     for i in _ignored_options:
         del _params[i]
@@ -492,7 +519,6 @@ def xkb_rules_names():
     }
     return _rn_dict
 
-
 def local_color_depth():
     """\
     Detect the current local screen's color depth.
@@ -538,6 +564,8 @@ def is_color_depth_ok(depth_session, depth_local):
         return True
     if ( ( depth_session == 24 or depth_session == 32 ) and ( depth_local == 24 or depth_local == 32 ) ):
         return True;
+    if ( ( depth_session == 16 or depth_session == 17 ) and ( depth_local == 16 or depth_local == 17 ) ):
+        return True;
     return False
 
 
@@ -558,13 +586,17 @@ def find_session_window(session_name):
         root = display.screen().root
 
         success = False
-        windowIDs = root.get_full_property(display.intern_atom('_NET_CLIENT_LIST'), Xlib.X.AnyPropertyType).value
-        for windowID in windowIDs:
-            window = display.create_resource_object('window', windowID)
-            name = window.get_wm_name()
-            if name is not None and session_name in name:
-                success = True
-                break
+        windowIDs_obj = root.get_full_property(display.intern_atom('_NET_CLIENT_LIST'), Xlib.X.AnyPropertyType)
+
+        if windowIDs_obj is not None:
+            windowIDs = windowIDs_obj.value
+
+            for windowID in windowIDs:
+                window = display.create_resource_object('window', windowID)
+                name = window.get_wm_name()
+                if name is not None and session_name in name:
+                    success = True
+                    break
 
         if success:
             return window
@@ -584,6 +616,50 @@ def find_session_window(session_name):
         return window
 
 
+def get_desktop_geometry():
+    """\
+    Get the geometry of the current screen's desktop by
+    wrapping around::
+
+        xprop -root '_NET_DESKTOP_GEOMETRY'
+
+    @return: a (<width>, <height>) tuple will be returned
+    @rtype: C{tuple}
+
+    """
+    if _X2GOCLIENT_OS != 'Windows':
+        p = subprocess.Popen(['xprop', '-root', '_NET_DESKTOP_GEOMETRY',], stdout=subprocess.PIPE, )
+        _paramval = p.stdout.read().split("=")
+        if len(_paramval) == 2:
+            _list = _paramval[1].rstrip('\n').split(',')
+            if len(_list) == 2:
+                return (_list[0].strip(), _list[1].strip())
+
+    return None
+
+def get_workarea_geometry():
+    """\
+    Get the geometry of the current screen's work area by
+    wrapping around::
+
+        xprop -root '_NET_WORKAREA'
+
+    @return: a (<width>, <height>) tuple will be returned
+    @rtype: C{tuple}
+
+    """
+    if _X2GOCLIENT_OS != 'Windows':
+        p = subprocess.Popen(['xprop', '-root', '_NET_WORKAREA',], stdout=subprocess.PIPE, )
+        _list = p.stdout.read().rstrip('\n').split(',')
+        if len(_list) == 4:
+            return (_list[2].strip(), _list[3].strip())
+        else:
+            return None
+    else:
+
+        return None
+
+
 def set_session_window_title(session_window, session_title):
     """\
     Set title of session window.
@@ -657,3 +733,65 @@ def merge_ordered_lists(l1, l2):
 
     return ordered_list
 
+def compare_versions(version_a, op, version_b):
+    """\
+    Compare <version_a> with <version_b> using operator <op>.
+    In the background C{distutils.version.LooseVersion} is
+    used for the comparison operation.
+
+    @param version_a: a version string
+    @type version_a: C{str}
+    @param op: an operator provide as string (e.g. '<', '>', '==', '>=' etc.)
+    @type op: C{str}
+    @param version_b: another version string that is to be compared with <version_a>
+    @type version_b: C{str}
+
+    """
+
+    ### FIXME: this comparison is not reliable with beta et al. version strings
+
+    ver_a = distutils.version.LooseVersion(version_a)
+    ver_b = distutils.version.LooseVersion(version_b)
+
+    return eval("ver_a %s ver_b" % op)
+
+class ProgressStatus(object):
+    """\
+    A simple progress status iterator class.
+
+    """
+    def __init__(self, progress_event, progress_func=range(0, 100, 10)):
+        """\
+        @param progress_event: a threading.Event() object that gets notified on progress
+        @type progress_event: C{obj}
+        @param progress_func: a function that delivers a value between 0 and 100 (progress percentage value)
+        @type progress_func: C{func}
+
+        """
+        self.ev = progress_event
+        self.progress_func = progress_func
+
+    def __iter__(self):
+        """\
+        Intialize the L{ProgresStatus} iterator object.
+
+        """
+        self.status = self.progress_func()
+        return self
+
+    def next(self):
+        """\
+        On each iteration wait for the progress event to get triggered from an outside
+        part of the application.
+
+        Once the event fires read the progress status from the progress retrieval function
+        and clear the event afterwards (so we wait for the next firing of the event).
+
+        """
+        if self.status < 100 and self.status != -1:
+            self.ev.wait()
+            self.status = self.progress_func()
+            self.ev.clear()
+            return self.status
+        else:
+            raise StopIteration
