@@ -1,24 +1,24 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2011 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2012 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
-# Python X2go is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
+# Python X2Go is free software; you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 #
-# Python X2go is distributed in the hope that it will be useful,
+# Python X2Go is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program; if not, write to the
 # Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
 """\
-Python X2go exceptions.
+Python X2Go exceptions.
 
 """
 __NAME__ = 'x2goexceptions-pylib'
@@ -29,7 +29,7 @@ import exceptions
 
 from defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
 
-# Python X2go Exceptions
+# Python X2Go Exceptions
 AuthenticationException = paramiko.AuthenticationException
 """inherited from Python Paramiko library"""
 PasswordRequiredException = paramiko.AuthenticationException
@@ -58,6 +58,7 @@ class X2goRevFwTunnelException(_X2goException): pass
 class X2goPrintException(_X2goException): pass
 class X2goPrintQueueException(_X2goException): pass
 class X2goPrintActionException(_X2goException): pass
+class X2goProxyException(_X2goException): pass
 class X2goMIMEboxActionException(_X2goException): pass
 class X2goMIMEboxQueueException(_X2goException): pass
 class X2goSSHProxyException(_X2goException): pass
