@@ -1,18 +1,18 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2011 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2012 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
-# Python X2go is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
+# Python X2Go is free software; you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 #
-# Python X2go is distributed in the hope that it will be useful,
+# Python X2Go is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program; if not, write to the
 # Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
@@ -36,12 +36,12 @@ import threading
 import gevent
 import copy
 
-# Python X2go modules
+# Python X2Go modules
 import log
 from defaults import X2GO_XCONFIG_CONFIGFILES as _X2GO_XCONFIG_CONFIGFILES
 from defaults import X2GO_CLIENTXCONFIG_DEFAULTS as _X2GO_CLIENTXCONFIG_DEFAULTS
 import inifiles
-
+import utils
 
 class X2goClientXConfig(inifiles.X2goIniFile):
     """\
@@ -72,7 +72,7 @@ class X2goClientXConfig(inifiles.X2goIniFile):
         @param defaults: a Python dictionary with configuration file defaults (use on your own risk)
         @type defaults: C{dict}
         @param logger: you can pass an L{X2goLogger} object to the L{X2goClientXConfig} constructor
-        @type logger: C{instance}
+        @type logger: C{obj}
         @param loglevel: if no L{X2goLogger} object has been supplied a new one will be
             constructed with the given loglevel
         @type loglevel: C{int}
@@ -85,6 +85,13 @@ class X2goClientXConfig(inifiles.X2goIniFile):
 
         inifiles.X2goIniFile.__init__(self, config_files, defaults=defaults, logger=logger, loglevel=loglevel)
 
+        _known_xservers = utils.merge_ordered_lists(self.defaultValues['XServers']['known_xservers'], self.known_xservers)
+
+        if _known_xservers != self.known_xservers:
+            self.update_value('XServers', 'known_xservers', _known_xservers)
+            self.write_user_config = True
+            self.write()
+
     def get_xserver_config(self, xserver_name):
         """\
         Retrieve the XServer configuration (from the xconfig file) for the given XServer application.
@@ -98,13 +105,16 @@ class X2goClientXConfig(inifiles.X2goIniFile):
         """
         _xserver_config = {}
         for option in self.iniConfig.options(xserver_name):
-            _xserver_config[option] = self.get(xserver_name, option, key_type=self.get_type(xserver_name, option))
+            try:
+                _xserver_config[option] = self.get(xserver_name, option, key_type=self.get_type(xserver_name, option))
+            except KeyError:
+                pass
         return _xserver_config
 
     @property
     def known_xservers(self):
         """\
-        Renders a list of XServers that are known to Python X2go.
+        Renders a list of XServers that are known to Python X2Go.
 
         """
         return self.get_value('XServers', 'known_xservers')
@@ -156,7 +166,7 @@ class X2goClientXConfig(inifiles.X2goIniFile):
         """\
         Detect if an XServer launch is really needed (or if we use an already running XServer instance).
         Equals C{True} if we have to launch an XServer before we can start/resume
-        X2go sessions.
+        X2Go sessions.
 
         """
         return not bool(self.running_xservers)
@@ -164,24 +174,79 @@ class X2goClientXConfig(inifiles.X2goIniFile):
     @property
     def preferred_xserver(self):
         """\
-        Renders a list of preferred XServer names (most preferred on top).
+        Returns a tuple of (<xserver_name>, <xserver_config>).
+
+        return: (<xserver_name>, <xserver_config>)
+        rtype: C{tuple}
 
         """
-        if self.xserver_launch_possible and self.xserver_launch_needed:
+        if self.xserver_launch_possible:
             return (self.installed_xservers[0], self.get_xserver_config(self.installed_xservers[0]))
         else:
             return None
 
+    @property
+    def preferred_xserver_names(self):
+        """\
+        Returns the list of preferred XServer names (most preferred first).
+
+        """
+        return self.installed_xservers
+
+    def detect_unused_xdisplay_port(self, xserver_name):
+        """\
+        Get an unused TCP/IP port for the to-be-launched X server and write it
+        to the user's X configuration file.
+
+        @param xserver_name: name of the XServer application
+        @type xserver_name: C{str}
+
+        """
+        _default_display = self.get_xserver_config(xserver_name)['display']
+        _last_display = self.get_xserver_config(xserver_name)['last_display']
+
+        try:
+            _default_xserver_port = int(_default_display.split(":")[1].split(".")[0]) + 6000
+            _last_xserver_port = int(_last_display.split(":")[1].split(".")[0]) + 6000
+
+            # try the last used $DISPLAY first...
+            if utils.detect_unused_port(preferred_port=_last_xserver_port) == _last_xserver_port:
+                _detect_xserver_port = _last_xserver_port
+
+            # then try the default $DISPLAY...
+            elif utils.detect_unused_port(preferred_port=_default_xserver_port) == _default_xserver_port:
+                _detect_xserver_port = _default_xserver_port
+
+            # otherwise use a detection algorithm to find a free TCP/IP port
+            else:
+                _xserver_port = _default_xserver_port +1
+                while utils.detect_unused_port(preferred_port=_xserver_port) != _xserver_port:
+                        _xserver_port += 1
+                _detect_xserver_port = _xserver_port
+
+            # if the port changed, let's write it to our configuration file
+            if _detect_xserver_port != _last_xserver_port:
+                _new_display = _last_display.replace(str(_last_xserver_port -6000), str(_detect_xserver_port -6000))
+                self.logger('cannot used configured X DISPLAY, the new available DISPLAY port %s has been detected' % _new_display, loglevel=log.loglevel_NOTICE)
+                self.update_value(xserver_name, 'last_display', _new_display)
+                _parameters = self.get_value(xserver_name, 'parameters')
+                _parameters[0] = ":%s" % (_detect_xserver_port -6000)
+                self.update_value(xserver_name, 'parameters', tuple(_parameters))  
+                self.write_user_config = True
+                self.write()
+
+        except TypeError:
+            pass
+
 
 class X2goXServer(threading.Thread):
     """
     This class is responsible for starting/stopping an external XServer application.
 
-    X2go applications require a running XServer on the client system. This class will
-    manage/handle the XServer while your X2go application is running.
+    X2Go applications require a running XServer on the client system. This class will
+    manage/handle the XServer while your X2Go application is running.
 
     """
-
     def __init__(self, xserver_name, xserver_config, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
         Initialize an XServer thread.
@@ -191,7 +256,7 @@ class X2goXServer(threading.Thread):
         @param xserver_config: XServer configuration node (as derived from L{X2goClientXConfig.get_xserver_config()}
         @type xserver_config: C{dict}
         @param logger: you can pass an L{X2goLogger} object to the L{X2goClientXConfig} constructor
-        @type logger: C{instance}
+        @type logger: C{obj}
         @param loglevel: if no L{X2goLogger} object has been supplied a new one will be
             constructed with the given loglevel
         @type loglevel: C{int}
@@ -212,13 +277,23 @@ class X2goXServer(threading.Thread):
 
         self.xserver_name = xserver_name
         self.xserver_config = xserver_config
-        if self.xserver_config.has_key('display'):
-            self.logger('settings DISPLAY environment variable to %s' % self.xserver_config['display'], loglevel=log.loglevel_NOTICE)
-            os.environ.update({'DISPLAY': str(self.xserver_config['display'])})
+        self.hProcess = None
+
+        if self.xserver_config.has_key('last_display'):
+
+            self.logger('setting DISPLAY environment variable to %s' % self.xserver_config['last_display'], loglevel=log.loglevel_NOTICE)
+            os.environ.update({'DISPLAY': str(self.xserver_config['last_display'])})
         threading.Thread.__init__(self)
         self.daemon = True
         self.start()
 
+    def __del__(self):
+        """\
+        Class destructor. Terminate XServer process.
+
+        """
+        self._terminate_xserver()
+
     def run(self):
         """\
         Start this L{X2goXServer} thread. This will launch the configured XServer application.
@@ -241,24 +316,31 @@ class X2goXServer(threading.Thread):
                                                 None,
                                                 si,
                                                )
-            (hProcess, hThread, processId, threadId) = p_info
+            (self.hProcess, hThread, processId, threadId) = p_info
 
         while self._keepalive:
             gevent.sleep(1)
 
+        self._terminate_xserver()
+
+    def _terminate_xserver(self):
+        """\
+        Terminate the runnint XServer process.
+
+        """
         self.logger('terminating running XServer ,,%s\'\'' % self.xserver_name, loglevel=log.loglevel_DEBUG)
 
-        if _X2GOCLIENT_OS == 'Windows':
+        if _X2GOCLIENT_OS == 'Windows' and self.hProcess is not None:
             try:
-                win32process.TerminateProcess(hProcess, 0)
+                win32process.TerminateProcess(self.hProcess, 0)
             except win32process.error:
-                pass
+                self.logger('XServer ,,%s\'\' could not be terminated.' % self.xserver_name, loglevel=log.loglevel_DEBUG)
 
     def stop_thread(self):
         """\
         A call to this method will stop the XServer application and do a cleanup afterwards.
 
         """
-        self.logger('stop_thread() method has been called', loglevel=log.loglevel_DEBUG)
         self._keepalive = False
+        self.logger('stop_thread() method has been called', loglevel=log.loglevel_DEBUG)
 
